@@ -131,6 +131,9 @@ BEGIN
 
   SHOW GLOBAL STATUS LIKE 'slave_open_temp_tables';
 
+  -- Show open connections/transactions in wsrep provider
+  SHOW STATUS LIKE 'wsrep_open%';
+
   -- Checksum system tables to make sure they have been properly
   -- restored after test
   -- skip mysql.proc however, as created timestamps may have been updated by mysqldump.test / mysql_ugprade.test
