@@ -143,7 +143,13 @@ sub collect_test_cases ($$$$) {
 
   if ( @$opt_cases )
   {
-    # A list of tests was specified on the command line
+    # A list of tests was specified on the command line.
+    # Among those, the tests which are not already collected will be
+    # collected and stored temporarily in an array of hashes pointed
+    # by the below reference. This array is eventually appeneded to
+    # the one having all collected test cases.
+    my $cmdline_cases;
+
     # Check that the tests specified was found
     # in at least one suite
     foreach my $test_name_spec ( @$opt_cases )
@@ -162,20 +168,56 @@ sub collect_test_cases ($$$$) {
       }
       if ( not $found )
       {
-	$sname= "main" if !$opt_reorder and !$sname;
-	mtr_error("Could not find '$tname' in '$suites' suite(s)") unless $sname;
-	# If suite was part of name, find it there, may come with combinations
-	my @this_case = collect_one_suite($sname, [ $tname ]);
-	if (@this_case)
+        if ( $sname )
         {
-	  push (@$cases, @this_case);
-	}
-	else
-	{
-	  mtr_error("Could not find '$tname' in '$sname' suite");
+	  # If suite was part of name, find it there, may come with combinations
+	  my @this_case = collect_one_suite($sname, [ $tname ]);
+
+          # If a test is specified multiple times on the command line, all
+          # instances of the test need to be picked. Hence, such tests are
+          # stored in the temporary array instead of adding them to $cases
+          # directly so that repeated tests are not run only once
+	  if (@this_case)
+          {
+	    push (@$cmdline_cases, @this_case);
+	  }
+	  else
+	  {
+	    mtr_error("Could not find '$tname' in '$sname' suite");
+          }
+        }
+        else
+        {
+          if ( !$opt_reorder )
+          {
+            # If --no-reorder is passed and if suite was not part of name,
+            # search in all the suites
+            foreach my $suite (split(",", $suites))
+            {
+              my @this_case = collect_one_suite($suite, [ $tname ]);
+              if ( @this_case )
+              {
+                push (@$cmdline_cases, @this_case);
+                $found= 1;
+              }
+              @this_case= collect_one_suite("i_".$suite, [ $tname ]);
+              if ( @this_case )
+              {
+                push (@$cmdline_cases, @this_case);
+                $found= 1;
+              }
+            }
+          }
+          if ( !$found )
+          {
+            mtr_error("Could not find '$tname' in '$suites' suite(s)");
+          }
         }
       }
     }
+    # Add test cases collected in the temporary array to the one
+    # containing all previously collected test cases
+    push (@$cases, @$cmdline_cases) if $cmdline_cases;
   }
 
   if ( $opt_reorder && !$quick_collect)
@@ -503,7 +545,17 @@ sub collect_one_suite($)
 	my $comb= {};
 	$comb->{name}= $group->name();
         foreach my $option ( $group->options() ) {
-	  push(@{$comb->{comb_opt}}, $option->option());
+        my $option_string = $option->option();
+        if ($option_string =~ m/--mtr[-_]result[-_]dir\s*=\s*(.+)/) {
+          mtr_verbose("The result dir was changed to $1");
+          $comb->{result_dir} = $1;
+        } elsif ($option_string =~ m/--mtr[-_]tests[-_]list\s*=\s*(.+)/) {
+          my @comb_tests_array = split (/,/, $1);
+          my %comb_tests_set = map { $_ => 1 } @comb_tests_array;
+          $comb->{tests_set} = \%comb_tests_set;
+        } else {
+          push(@{$comb->{comb_opt}}, $option->option());
+        }
 	}
 	push(@combinations, $comb);
       }
@@ -525,6 +577,8 @@ sub collect_one_suite($)
 
 	  next if ( $test->{'skip'} );
 
+    next if ($comb->{tests_set} and not
+             $comb->{tests_set}->{$test->{shortname}});
 	  # Skip this combination if the values it provides
 	  # already are set in master_opt or slave_opt
 	  if (My::Options::is_set($test->{master_opt}, $comb->{comb_opt}) &&
@@ -546,6 +600,16 @@ sub collect_one_suite($)
 	  push(@{$new_test->{master_opt}}, @{$comb->{comb_opt}});
 	  push(@{$new_test->{slave_opt}}, @{$comb->{comb_opt}});
 
+    if ($comb->{result_dir}) {
+      my $result_file = "$suitedir/$comb->{result_dir}/".
+                        "$new_test->{shortname}.result";
+      if (-f $result_file) {
+        $new_test->{result_file} = $result_file;
+      } else {
+        $new_test->{record_file} =  $result_file;
+      }
+    }
+
 	  # Add combination name short name
 	  $new_test->{combination}= $comb->{name};
 
