@@ -774,7 +774,7 @@ int my_load_defaults(const char *conf_file, const char **groups,
   my_message_local(ERROR_LEVEL,
                    "Fatal error in defaults handling. Program aborted!");
   exit(1);
-  return 0;					/* Keep compiler happy */
+  DBUG_RETURN(0); /* Keep compiler happy */
 }
 
 
