@@ -59,6 +59,8 @@ galera::Wsdb::~Wsdb()
     // and don't clean up to let valgrind etc to detect leaks.
 #ifndef NDEBUG
     std::cerr << *this;
+    assert(trx_map_.size() == 0);
+    assert(conn_map_.size() == 0);
 #else
     for_each(trx_map_.begin(), trx_map_.end(), Unref2nd<TrxMap::value_type>());
     for_each(conn_trx_map_.begin(),
@@ -74,6 +76,12 @@ galera::Wsdb::find_trx(wsrep_trx_id_t const trx_id)
     gu::Lock lock(trx_mutex_);
 
     galera::TrxHandle* trx;
+    /* trx-id = 0 is safe-guard condition.
+    trx-id is generally assigned from thd->query-id
+    and query-id default is 0. If background thread
+    try to assign set wsrep_next_trx_id before setting
+    query-id we will hit the said assert. */
+    assert(trx_id != 0);
 
     if (trx_id != wsrep_trx_id_t(-1))
     {
@@ -223,15 +231,6 @@ void galera::Wsdb::discard_conn_query(wsrep_conn_id_t conn_id)
     if ((i = conn_map_.find(conn_id)) != conn_map_.end())
     {
         i->second.assign_trx(0);
-    }
-}
-
-void galera::Wsdb::discard_conn(wsrep_conn_id_t conn_id)
-{
-    gu::Lock lock(conn_mutex_);
-    ConnMap::iterator i;
-    if ((i = conn_map_.find(conn_id)) != conn_map_.end())
-    {
         conn_map_.erase(i);
     }
 }
