@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2014 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2017 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -24,6 +24,9 @@
 #include <unistd.h>
 #include <stdio.h>
 
+#include <wsrep_api.h>
+
+
 /*! @typedef @brief Sequence number type. */
 typedef int64_t gcs_seqno_t;
 
@@ -280,9 +283,11 @@ extern long gcs_resume_recv (gcs_conn_t* conn);
  * After action with this seqno is applied, this thread is guaranteed to see
  * all the changes made by the client, even on other nodes.
  *
- * @return global sequence number or negative error code
+ * @retval 0       success
+ * @retval -EPERM  operation not permitted (in NON_PRIMARY state)
+ * @retval -EAGAIN operation may be retried later (in transient state)
  */
-extern gcs_seqno_t gcs_caused(gcs_conn_t* conn);
+extern long gcs_caused (gcs_conn_t* conn, gcs_seqno_t& seqno);
 
 /*! @brief Sends state transfer request
  * Broadcasts state transfer request which will be passed to one of the
@@ -431,7 +436,8 @@ struct gcs_stats
     double    recv_q_len_avg; //! average recv queue length per queued action
     long long fc_paused_ns;   //! total nanoseconds spent in paused state
     double    fc_paused_avg;  //! faction of time paused due to flow control
-    long long fc_sent;        //! flow control stops sent
+    long long fc_ssent;       //! flow control stops sent
+    long long fc_csent;       //! flow control conts sent
     long long fc_received;    //! flow control stops received
     size_t    recv_q_size;    //! current recv queue size
     int       recv_q_len;     //! current recv queue length
@@ -455,6 +461,9 @@ void gcs_get_status(gcs_conn_t* conn, gu::Status& status);
 
 extern void gcs_join_notification(gcs_conn_t *conn);
 
+extern void
+gcs_fetch_pfs_info (gcs_conn_t* conn, wsrep_node_info_t* entries, uint32_t size);
+
 /*! A node with this name will be treated as a stateless arbitrator */
 #define GCS_ARBITRATOR_NAME "garb"
 
