@@ -335,11 +335,18 @@ public:
         pthread_join(thd_, 0);
         {
             gcomm::Critical<Protonet> crit(*net_);
-            log_info << "gcomm: closing backend";
-            tp_->close(error_ != 0 || force == true);
-            gcomm::disconnect(tp_, this);
-            delete tp_;
-            tp_ = 0;
+            if (tp_ == 0)
+            {
+                log_info << "gcomm: backend closed already";
+            }
+            else
+            {
+                log_info << "gcomm: closing backend";
+                tp_->close(error_ != 0 || force == true);
+                gcomm::disconnect(tp_, this);
+                delete tp_;
+                tp_ = 0;
+            }
         }
         const Message* msg;
 
@@ -742,6 +749,7 @@ static GCS_BACKEND_RECV_FN(gcomm_recv)
             if (cm_size <= msg->buf_len)
             {
                 memcpy(msg->buf, cm, cm_size);
+                msg->size = cm_size;
                 recv_buf.pop_front();
                 msg->type = GCS_MSG_COMPONENT;
             }
@@ -907,6 +915,8 @@ GCS_BACKEND_PARAM_SET_FN(gcomm_param_set)
         return -EBADFD;
     }
 
+    Protolay::sync_param_cb_t sync_param_cb;
+
     GCommConn& conn(*ref.get());
     try
     {
@@ -916,15 +926,11 @@ GCS_BACKEND_PARAM_SET_FN(gcomm_param_set)
             return -ECONNABORTED;
         }
 
-        if (conn.get_pnet().set_param(key, value) == false)
+        if (conn.get_pnet().set_param(key, value, sync_param_cb) == false)
         {
             log_debug << "param " << key << " not recognized";
             return 1;
         }
-        else
-        {
-            return 0;
-        }
     }
     catch (gu::Exception& e)
     {
@@ -947,6 +953,13 @@ GCS_BACKEND_PARAM_SET_FN(gcomm_param_set)
         log_fatal << "gcomm param set: caught unknown exception";
         return -ENOTRECOVERABLE;
     }
+
+    if (!sync_param_cb.empty()) 
+    {
+        sync_param_cb();
+    }
+  
+    return 0;
 }
 
 
