@@ -124,13 +124,13 @@ rm -rf "$KEYRING_FILE"
 WSREP_LOG_DIR=${WSREP_LOG_DIR:-""}
 # if WSREP_LOG_DIR env. variable is not set, try to get it from my.cnf
 if [ -z "$WSREP_LOG_DIR" ]; then
-    WSREP_LOG_DIR=$(parse_cnf mysqld-5.6 innodb_log_group_home_dir "")
+    WSREP_LOG_DIR=$(parse_cnf mysqld-5.6 innodb-log-group-home-dir "")
 fi
 if [ -z "$WSREP_LOG_DIR" ]; then
-    WSREP_LOG_DIR=$(parse_cnf mysqld innodb_log_group_home_dir "")
+    WSREP_LOG_DIR=$(parse_cnf mysqld innodb-log-group-home-dir "")
 fi
 if [ -z "$WSREP_LOG_DIR" ]; then
-    WSREP_LOG_DIR=$(parse_cnf server innodb_log_group_home_dir "")
+    WSREP_LOG_DIR=$(parse_cnf server innodb-log-group-home-dir "")
 fi
 
 if [ -n "$WSREP_LOG_DIR" ]; then
@@ -148,8 +148,16 @@ fi
 #         --exclude '*.[0-9][0-9][0-9][0-9][0-9][0-9]' --exclude '*.index')
 
 # New filter - exclude everything except dirs (schemas) and innodb files
-FILTER=(-f '- /lost+found' -f '- /.fseventsd' -f '- /.Trashes'
-        -f '+ /wsrep_sst_binlog.tar' -f '+ /ib_lru_dump' -f '+ /*.ibd' -f '+ /ibdata*' -f '+ /*/' -f '- /*')
+FILTER=(-f '- /lost+found'
+        -f '- /.fseventsd'
+        -f '- /.Trashes'
+        -f '+ /wsrep_sst_binlog.tar'
+        -f '+ /ib_lru_dump'
+        -f '+ /ibdata*'
+        -f '+ /undo*'
+        -f '+ /*.ibd'
+        -f '+ /*/'
+        -f '- /*')
 
 if [ "$WSREP_SST_OPT_ROLE" = "donor" ]
 then
