@@ -124,7 +124,8 @@ void append_user(THD *thd, String *str, LEX_USER *user, bool comma= true,
   }
 }
 
-void append_user_new(THD *thd, String *str, LEX_USER *user, bool comma= true)
+void append_user_new(THD *thd, String *str, LEX_USER *user, bool comma,
+                     bool hide_password_hash)
 {
   String from_user(user->user.str, user->user.length, system_charset_info);
   String from_plugin(user->plugin.str, user->plugin.length, system_charset_info);
@@ -174,7 +175,8 @@ void append_user_new(THD *thd, String *str, LEX_USER *user, bool comma= true)
       if (user->auth.length > 0)
       {
         str->append(STRING_WITH_LEN(" AS "));
-        if (thd->lex->contains_plaintext_password)
+        if (thd->lex->contains_plaintext_password ||
+            hide_password_hash)
         {
           str->append("'");
           str->append(STRING_WITH_LEN("<secret>"));
@@ -273,15 +275,17 @@ int check_change_password(THD *thd, const char *host, const char *user,
 /**
   Auxiliary function for constructing CREATE USER sql for a given user.
 
-  @param thd          Thread context
-  @param user_name    user for which the sql should be constructed.
+  @param thd                    Thread context
+  @param user_name              user for which the sql should be constructed.
+  @param are_both_users_same    If the command is issued for self or not.
 
   @retval
     0         OK.
     1         Error.
  */
 
-bool mysql_show_create_user(THD *thd, LEX_USER *user_name)
+bool mysql_show_create_user(THD *thd, LEX_USER *user_name,
+                            bool are_both_users_same)
 {
   int error= 0;
   ACL_USER *acl_user;
@@ -295,8 +299,17 @@ bool mysql_show_create_user(THD *thd, LEX_USER *user_name)
   List<Item> field_list;
   String sql_text(buff,sizeof(buff),system_charset_info);
   LEX_ALTER alter_info;
+  bool hide_password_hash= false;
 
   DBUG_ENTER("mysql_show_create_user");
+  if (are_both_users_same)
+  {
+    TABLE_LIST t1;
+    t1.init_one_table(C_STRING_WITH_LEN("mysql"), C_STRING_WITH_LEN("user"),
+                      "user", TL_READ);
+    hide_password_hash= check_table_access(thd, SELECT_ACL, &t1, false,
+                                           UINT_MAX, true);
+  }
 
   mysql_mutex_lock(&acl_cache->lock);
   if (!(acl_user= find_acl_user(user_name->host.str, user_name->user.str, TRUE)))
@@ -365,7 +378,7 @@ bool mysql_show_create_user(THD *thd, LEX_USER *user_name)
   }
   sql_text.length(0);
   lex->users_list.push_back(user_name);
-  mysql_rewrite_create_alter_user(thd, &sql_text);
+  mysql_rewrite_create_alter_user(thd, &sql_text, NULL, hide_password_hash);
   /* send the result row to client */
   protocol->start_row();
   protocol->store(sql_text.ptr(),sql_text.length(),sql_text.charset());
@@ -790,15 +803,19 @@ bool change_password(THD *thd, const char *host, const char *user,
   thd->lex->alter_password.account_locked= false;
   thd->lex->alter_password.update_password_expired_fields= false;
 
+
   /*
-    When @@log-backward-compatible-user-definitions variable is ON
-    and its a slave thread, then the password is already hashed. So
-    do not generate another hash.
-  */
-  if (opt_log_builtin_as_identified_by_password &&
-      thd->slave_thread)
+    In case its a slave thread or a binlog applier thread, the password
+    is already hashed. Do not generate another hash!
+   */
+  if (thd->slave_thread || thd->is_binlog_applier())
+  {
+    /* Password is in hash form */
+    combo->uses_authentication_string_clause= true;
+    /* Password is not plain text */
     combo->uses_identified_by_clause= false;
-    
+  }
+
   if (set_and_validate_user_attributes(thd, combo, what_to_set,
                                        true, "SET PASSWORD"))
   {
@@ -807,7 +824,7 @@ bool change_password(THD *thd, const char *host, const char *user,
     goto end;
   }
 
-  ret= replace_user_table(thd, table, combo, 0, false, true, what_to_set);
+  ret= replace_user_table(thd, table, combo, 0, false, false, what_to_set);
   if (ret)
   {
     mysql_mutex_unlock(&acl_cache->lock);
@@ -2019,7 +2036,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
     }
 
     /* update the mysql.user table */
-    int ret= replace_user_table(thd, table, user_from, 0, false, true,
+    int ret= replace_user_table(thd, table, user_from, 0, false, false,
                                 what_to_alter);
     if (ret)
     {
