@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -148,13 +148,19 @@ Channel_info* Per_thread_connection_handler::block_until_new_connection()
 
     if (kill_blocked_pthreads_flag)
       mysql_cond_signal(&COND_flush_thread_cache);
-    else if (!abort_loop && wake_pthread)
+    else if (wake_pthread)
     {
       wake_pthread--;
-      DBUG_ASSERT(!waiting_channel_info_list->empty());
-      new_conn= waiting_channel_info_list->front();
-      waiting_channel_info_list->pop_front();
-      DBUG_PRINT("info", ("waiting_channel_info_list->pop %p", new_conn));
+      if (!waiting_channel_info_list->empty())
+      {
+        new_conn = waiting_channel_info_list->front();
+        waiting_channel_info_list->pop_front();
+        DBUG_PRINT("info", ("waiting_channel_info_list->pop %p", new_conn));
+      }
+      else
+      {
+        DBUG_ASSERT(0);
+      }
     }
   }
   mysql_mutex_unlock(&LOCK_thread_cache);
@@ -320,7 +326,9 @@ extern "C" void *handle_connection(void *arg)
     thd->release_resources();
 
     // Clean up errors now, before possibly waiting for a new connection.
-    ERR_remove_state(0);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    ERR_remove_thread_state(0);
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
     thd_manager->remove_thd(thd);
     Connection_handler_manager::dec_connection_count(extra_port_connection);
@@ -361,6 +369,15 @@ extern "C" void *handle_connection(void *arg)
     if (channel_info == NULL)
       break;
     pthread_reused= true;
+    if (abort_loop)
+    {
+      // Close the channel and exit as server is undergoing shutdown.
+      channel_info->send_error_and_close_channel(ER_SERVER_SHUTDOWN, 0, false);
+      delete channel_info;
+      channel_info = NULL;
+      Connection_handler_manager::dec_connection_count(extra_port_connection);
+      break;
+    }
   }
 
   my_thread_end();
@@ -379,16 +396,6 @@ void Per_thread_connection_handler::kill_blocked_pthreads()
     mysql_cond_wait(&COND_flush_thread_cache, &LOCK_thread_cache);
   }
   kill_blocked_pthreads_flag--;
-
-  // Drain off the channel info list.
-  while (!waiting_channel_info_list->empty())
-  {
-    Channel_info* channel_info= waiting_channel_info_list->front();
-    waiting_channel_info_list->pop_front();
-    // close the channel.
-    channel_info->send_error_and_close_channel(ER_SERVER_SHUTDOWN, 0, false);
-    delete channel_info;
-  }
   mysql_mutex_unlock(&LOCK_thread_cache);
 }
 
