@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -142,10 +142,15 @@ THD *Gtid_table_access_context::create_thd()
   */
   lex_start(thd);
   mysql_reset_thd_for_next_command(thd);
-
+  thd->set_skip_readonly_check();
   return(thd);
 }
 
+void Gtid_table_access_context::drop_thd(THD *thd)
+{
+  thd->reset_skip_readonly_check();
+  System_table_access::drop_thd(thd);
+}
 
 void Gtid_table_access_context::before_open(THD* thd)
 {
@@ -202,12 +207,25 @@ bool Gtid_table_access_context::init(THD **thd, TABLE **table, bool is_write)
 }
 
 
-void Gtid_table_access_context::deinit(THD *thd, TABLE *table,
+bool Gtid_table_access_context::deinit(THD *thd, TABLE *table,
                                        bool error, bool need_commit)
 {
   DBUG_ENTER("Gtid_table_access_context::deinit");
 
-  this->close_table(thd, table, &m_backup, 0 != error, need_commit);
+  bool err;
+  err= this->close_table(thd, table, &m_backup, 0 != error, need_commit);
+
+  /*
+    If err is true this means that there was some problem during
+    FLUSH LOGS commit phase.
+  */
+  if (err)
+  {
+    my_printf_error(ER_ERROR_DURING_FLUSH_LOGS, ER(ER_ERROR_DURING_FLUSH_LOGS),
+                    MYF(ME_FATALERROR), err);
+    sql_print_error(ER(ER_ERROR_DURING_FLUSH_LOGS), err);
+    DBUG_RETURN(err);
+  }
 
   /*
     If Gtid is inserted through Attachable_trx_rw its has been done
@@ -226,7 +244,7 @@ void Gtid_table_access_context::deinit(THD *thd, TABLE *table,
   if (m_drop_thd_object)
     this->drop_thd(m_drop_thd_object);
 
-  DBUG_VOID_RETURN;
+  DBUG_RETURN(err);
 }
 
 
@@ -455,7 +473,10 @@ int Gtid_table_persistor::save(const Gtid_set *gtid_set)
   ret= error= save(table, gtid_set);
 
 end:
-  table_access_ctx.deinit(thd, table, 0 != error, true);
+  const int deinit_ret= table_access_ctx.deinit(thd, table, 0 != error, true);
+
+  if (!ret && deinit_ret)
+    ret= -1;
 
   /* Notify compression thread to compress gtid_executed table. */
   if (error == 0 && DBUG_EVALUATE_IF("dont_compress_gtid_table", 0, 1))
@@ -849,6 +870,12 @@ extern "C" void *compress_gtid_table(void *p_thd)
   DBUG_ENTER("compress_gtid_table");
 
   init_thd(&thd);
+  /*
+    Gtid table compression thread should ignore 'read-only' and
+    'super_read_only' options so that it can update 'mysql.gtid_executed'
+    replication repository tables.
+  */
+  thd->set_skip_readonly_check();
   for (;;)
   {
     mysql_mutex_lock(&LOCK_compress_gtid_table);
@@ -884,6 +911,7 @@ extern "C" void *compress_gtid_table(void *p_thd)
   }
 
   mysql_mutex_unlock(&LOCK_compress_gtid_table);
+  thd->reset_skip_readonly_check();
   deinit_thd(thd);
   DBUG_LEAVE;
   my_thread_end();
