@@ -870,7 +870,9 @@ public:
     if (deferred_events)
     {
       delete deferred_events;
+#ifdef WITH_WSREP
       deferred_events= NULL;
+#endif /* WITH_WSREP */
     }
   };
    
@@ -1338,6 +1340,13 @@ public:
     @param  thd a reference to THD
   */
   void detach_engine_ha_data(THD *thd);
+  /**
+    Reattaches the engine ha_data to THD. The fact
+    is memorized in @c is_engine_ha_detached flag.
+
+    @param  thd a reference to THD
+  */
+  void reattach_engine_ha_data(THD *thd);
   /**
     Drops the engine ha_data flag when it is up.
     The method is run at execution points of the engine ha_data
