@@ -37,6 +37,9 @@
 #include "opt_trace.h"         // opt_trace_disable_etc
 
 #include <my_user.h>           // parse_user
+#ifdef WITH_WSREP
+#include "wsrep_thd.h"
+#endif /* WITH_WSREP */
 #include "mysql/psi/mysql_statement.h"
 #include "mysql/psi/mysql_sp.h"
 
@@ -804,6 +807,36 @@ bool sp_head::execute(THD *thd, bool merge_da_on_success)
     thd->m_statement_psi= parent_locker;
 #endif
 
+#ifdef WITH_WSREP
+    if (m_type == SP_TYPE_PROCEDURE ||
+        m_type == SP_TYPE_EVENT)
+    {
+      mysql_mutex_lock(&thd->LOCK_wsrep_thd);
+      if (thd->wsrep_conflict_state == MUST_REPLAY)
+      {
+        wsrep_replay_sp_transaction(thd);
+        err_status= thd->get_stmt_da()->is_set();
+        thd->wsrep_conflict_state= NO_CONFLICT;
+      }
+      else if (thd->wsrep_conflict_state == ABORTED ||
+               thd->wsrep_conflict_state == CERT_FAILURE)
+      {
+        /*
+          If the statement execution was BF aborted or was aborted
+          due to certification failure, clean up transaction here
+          and reset conflict state to NO_CONFLICT and thd->killed
+          to THD::NOT_KILLED. Error handling is done based on err_status
+          below. Error must have been raised by wsrep hton code before
+          entering here.
+         */
+        DBUG_ASSERT(err_status);
+        DBUG_ASSERT(thd->get_stmt_da()->is_error());
+        thd->wsrep_conflict_state= NO_CONFLICT;
+        thd->killed= THD::NOT_KILLED;
+      }
+      mysql_mutex_unlock(&thd->LOCK_wsrep_thd);
+    }
+#endif /* WITH_WSREP */
     thd->m_digest= parent_digest;
 
     thd->enable_slow_log= save_enable_slow_log;
@@ -2191,7 +2224,7 @@ void sp_head::add_used_tables_to_table_list(THD *thd,
 bool sp_head::check_show_access(THD *thd, bool *full_access)
 {
   TABLE_LIST tables;
-  memset(static_cast<void*>(&tables), 0, sizeof(tables));
+
   tables.db= (char*) "mysql";
   tables.table_name= tables.alias= (char*) "proc";
 
@@ -2254,6 +2287,8 @@ void sp_parser_data::start_parsing_sp_body(THD *thd, sp_head *sp)
   m_saved_free_list= thd->free_list;
 
   thd->mem_root= sp->get_persistent_mem_root();
+  set_memroot_max_capacity(thd->mem_root, m_saved_memroot->max_capacity);
+  set_memroot_error_reporting(thd->mem_root, m_saved_memroot->error_for_capacity_exceeded);
   thd->free_list= NULL;
 }
 
