@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -136,13 +136,6 @@ bool Sql_cmd_delete::mysql_delete(THD *thd, ha_rows limit)
     DBUG_RETURN(true);
 
   const_cond= (!conds || conds->const_item());
-  if (safe_update && const_cond)
-  {
-    my_message(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE,
-               ER(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE), MYF(0));
-    DBUG_RETURN(TRUE);
-  }
-
   const_cond_result= const_cond && (!conds || conds->val_int());
   if (thd->is_error())
   {
@@ -192,6 +185,14 @@ bool Sql_cmd_delete::mysql_delete(THD *thd, ha_rows limit)
       goto exit_without_my_ok;
     }
 
+    /* Do not allow deletion of all records if safe_update is set. */
+    if (safe_update)
+    {
+      my_error(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE, MYF(0),
+               thd->get_stmt_da()->get_first_condition_message());
+      DBUG_RETURN(true);
+    }
+
     DBUG_PRINT("debug", ("Trying to use delete_all_rows()"));
     if (!(error=table->file->ha_delete_all_rows()))
     {
@@ -291,7 +292,8 @@ bool Sql_cmd_delete::mysql_delete(THD *thd, ha_rows limit)
       QUICK_SELECT_I *qck;
       zero_rows= test_quick_select(thd, keys_to_use, 0, limit, safe_update,
                                    ORDER::ORDER_NOT_RELEVANT, &qep_tab,
-                                   conds, &needed_reg_dummy, &qck) < 0;
+                                   conds, &needed_reg_dummy, &qck,
+                                   qep_tab.table()->force_index) < 0;
       qep_tab.set_quick(qck);
     }
     if (zero_rows)
@@ -321,13 +323,22 @@ bool Sql_cmd_delete::mysql_delete(THD *thd, ha_rows limit)
   /* If running in safe sql mode, don't allow updates without keys */
   if (table->quick_keys.is_clear_all())
   {
-    thd->server_status|=SERVER_QUERY_NO_INDEX_USED;
-    if (safe_update && !using_limit)
+    thd->server_status|= SERVER_QUERY_NO_INDEX_USED;
+
+    /*
+      Safe update error isn't returned if:
+      1) It is  an EXPLAIN statement OR
+      2) LIMIT is present.
+
+      Append the first warning (if any) to the error message. This allows the
+      user to understand why index access couldn't be chosen.
+    */
+    if (!thd->lex->is_explain() && safe_update &&  !using_limit)
     {
       free_underlaid_joins(thd, select_lex);
-      my_message(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE,
-                 ER(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE), MYF(0));
-      DBUG_RETURN(TRUE);
+      my_error(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE, MYF(0),
+               thd->get_stmt_da()->get_first_condition_message());
+      DBUG_RETURN(true);
     }
   }
 
@@ -668,7 +679,6 @@ bool Sql_cmd_delete::mysql_prepare_delete(THD *thd)
     List<Item>   fields;
     List<Item>   all_fields;
 
-    memset(static_cast<void*>(&tables), 0, sizeof(tables));
     tables.table = table_list->table;
     tables.alias = table_list->alias;
 
