@@ -1,20 +1,13 @@
-/* Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
@@ -25,32 +18,20 @@
 
 #include <my_global.h>
 #include <vector>
-
-/**
-  the following #define adds server-only members to enum_mysql_show_type,
-  that is defined in plugin.h.
-*/
-#define SHOW_always_last SHOW_KEY_CACHE_LONG, \
-            SHOW_KEY_CACHE_LONGLONG, SHOW_LONG_STATUS, SHOW_DOUBLE_STATUS, \
-            SHOW_HAVE, SHOW_MY_BOOL, SHOW_HA_ROWS, SHOW_SYS, \
-            SHOW_LONG_NOFLUSH, SHOW_LONGLONG_STATUS, SHOW_LEX_STRING, \
-            SHOW_SIGNED_LONG
 #include <mysql/plugin.h>
-#undef SHOW_always_last
 
-#include "m_string.h"                       /* LEX_STRING */
+#include "mysql/mysql_lex_string.h"         // LEX_STRING
 #include "my_alloc.h"                       /* MEM_ROOT */
 #include "my_getopt.h"                      /* my_option */
+#include "sql_const.h"                      /* SHOW_COMP_OPTION */
 
-class sys_var;
-enum SHOW_COMP_OPTION { SHOW_OPTION_YES, SHOW_OPTION_NO, SHOW_OPTION_DISABLED};
-enum enum_plugin_load_option { PLUGIN_OFF, PLUGIN_ON, PLUGIN_FORCE,
-  PLUGIN_FORCE_PLUS_PERMANENT };
 extern const char *global_plugin_typelib_names[];
 extern mysql_mutex_t LOCK_plugin_delete;
 
 #include <my_sys.h>
 #include "sql_list.h"
+#include "sql_cmd.h"
+#include "sql_plugin_ref.h"
 
 #ifdef DBUG_OFF
 #define plugin_ref_to_int(A) A
@@ -67,9 +48,6 @@ extern mysql_mutex_t LOCK_plugin_delete;
 #define PLUGIN_INIT_SKIP_PLUGIN_TABLE    2
 #define PLUGIN_INIT_SKIP_INITIALIZATION  4
 
-#define INITIAL_LEX_PLUGIN_LIST_SIZE    16
-
-typedef enum enum_mysql_show_type SHOW_TYPE;
 typedef struct st_mysql_show_var SHOW_VAR;
 typedef struct st_mysql_lex_string LEX_STRING;
 
@@ -100,77 +78,102 @@ struct st_plugin_dl
   uint ref_count;            /* number of plugins loaded from the library */
 };
 
-/* A handle of a plugin */
 
-struct st_plugin_int
+/**
+   This class implements the INSTALL PLUGIN statement.
+*/
+
+class Sql_cmd_install_plugin : public Sql_cmd
 {
-  LEX_STRING name;
-  struct st_mysql_plugin *plugin;
-  struct st_plugin_dl *plugin_dl;
-  uint state;
-  uint ref_count;               /* number of threads using the plugin */
-  void *data;                   /* plugin type specific, e.g. handlerton */
-  MEM_ROOT mem_root;            /* memory for dynamic plugin structures */
-  sys_var *system_vars;         /* server variables for this plugin */
-  enum enum_plugin_load_option load_option; /* OFF, ON, FORCE, F+PERMANENT */
+public:
+  Sql_cmd_install_plugin(const LEX_STRING& comment,
+                         const LEX_STRING& ident)
+  : m_comment(comment), m_ident(ident)
+  { }
+
+  virtual enum_sql_command sql_command_code() const
+  { return SQLCOM_INSTALL_PLUGIN; }
+
+  /**
+    Install a new plugin by inserting a row into the
+    mysql.plugin table, creating a cache entry and
+    initializing plugin's internal data.
+
+    @param thd  Thread context
+
+    @returns false if success, true otherwise
+  */
+  virtual bool execute(THD *thd);
+
+private:
+  LEX_STRING m_comment;
+  LEX_STRING m_ident;
 };
 
 
-/*
-  See intern_plugin_lock() for the explanation for the
-  conditionally defined plugin_ref type
+/**
+   This class implements the UNINSTALL PLUGIN statement.
 */
-#ifdef DBUG_OFF
-typedef struct st_plugin_int *plugin_ref;
-#define plugin_decl(pi) ((pi)->plugin)
-#define plugin_dlib(pi) ((pi)->plugin_dl)
-#define plugin_data(pi,cast) ((cast)((pi)->data))
-#define plugin_name(pi) (&((pi)->name))
-#define plugin_state(pi) ((pi)->state)
-#define plugin_load_option(pi) ((pi)->load_option)
-#define plugin_equals(p1,p2) ((p1) == (p2))
-#else
-typedef struct st_plugin_int **plugin_ref;
-#define plugin_decl(pi) ((pi)[0]->plugin)
-#define plugin_dlib(pi) ((pi)[0]->plugin_dl)
-#define plugin_data(pi,cast) ((cast)((pi)[0]->data))
-#define plugin_name(pi) (&((pi)[0]->name))
-#define plugin_state(pi) ((pi)[0]->state)
-#define plugin_load_option(pi) ((pi)[0]->load_option)
-#define plugin_equals(p1,p2) ((p1) && (p2) && (p1)[0] == (p2)[0])
-#endif
+
+class Sql_cmd_uninstall_plugin : public Sql_cmd
+{
+public:
+  explicit Sql_cmd_uninstall_plugin(const LEX_STRING& comment)
+  : m_comment(comment)
+  { }
+
+  virtual enum_sql_command sql_command_code() const
+  { return SQLCOM_UNINSTALL_PLUGIN; }
+
+  /**
+    Uninstall a plugin by removing a row from the
+    mysql.plugin table, deleting a cache entry and
+    deinitializing plugin's internal data.
+
+    @param thd  Thread context
+
+    @returns false if success, true otherwise
+  */
+  virtual bool execute(THD *thd);
+
+private:
+  LEX_STRING m_comment;
+};
+
 
 typedef int (*plugin_type_init)(struct st_plugin_int *);
 
 extern I_List<i_string> *opt_plugin_load_list_ptr;
+extern I_List<i_string> *opt_early_plugin_load_list_ptr;
 extern char *opt_plugin_dir_ptr;
 extern char opt_plugin_dir[FN_REFLEN];
 extern const LEX_STRING plugin_type_names[];
 
-extern int plugin_init(int *argc, char **argv, int init_flags);
+extern bool plugin_register_early_plugins(int *argc, char **argv, int flags);
+extern bool plugin_register_builtin_and_init_core_se(int *argc, char **argv);
+extern bool plugin_register_dynamic_and_init_all(int *argc,
+                                                 char **argv, int init_flags);
 extern void plugin_shutdown(void);
 extern void memcached_shutdown(void);
 void add_plugin_options(std::vector<my_option> *options, MEM_ROOT *mem_root);
-extern bool plugin_is_ready(const LEX_STRING *name, int type);
+extern bool plugin_is_ready(const LEX_CSTRING &name, int type);
 #define my_plugin_lock_by_name(A,B,C) plugin_lock_by_name(A,B,C)
 #define my_plugin_lock_by_name_ci(A,B,C) plugin_lock_by_name(A,B,C)
 #define my_plugin_lock(A,B) plugin_lock(A,B)
 #define my_plugin_lock_ci(A,B) plugin_lock(A,B)
 extern plugin_ref plugin_lock(THD *thd, plugin_ref *ptr);
-extern plugin_ref plugin_lock_by_name(THD *thd, const LEX_STRING *name,
+extern plugin_ref plugin_lock_by_name(THD *thd, const LEX_CSTRING &name,
                                       int type);
 extern void plugin_unlock(THD *thd, plugin_ref plugin);
-extern void plugin_unlock_list(THD *thd, plugin_ref *list, uint count);
-extern bool mysql_install_plugin(THD *thd, const LEX_STRING *name,
-                                 const LEX_STRING *dl);
-extern bool mysql_uninstall_plugin(THD *thd, const LEX_STRING *name);
+extern void plugin_unlock_list(THD *thd, plugin_ref *list, size_t count);
 extern bool plugin_register_builtin(struct st_mysql_plugin *plugin);
 extern void plugin_thdvar_init(THD *thd, bool enable_plugins);
-extern void plugin_thdvar_cleanup(THD *thd);
+extern void plugin_thdvar_cleanup(THD *thd, bool enable_plugins);
 extern "C" void plugin_thdvar_safe_update(THD *thd, st_mysql_sys_var *var,
                                           char **dest, const char *value);
 extern SHOW_COMP_OPTION plugin_status(const char *name, size_t len, int type);
 extern bool check_valid_path(const char *path, size_t length);
+extern void alloc_and_copy_thd_dynamic_variables(THD *thd, bool global_lock);
 
 typedef my_bool (plugin_foreach_func)(THD *thd,
                                       plugin_ref plugin,
@@ -178,12 +181,16 @@ typedef my_bool (plugin_foreach_func)(THD *thd,
 #define plugin_foreach(A,B,C,D) plugin_foreach_with_mask(A,B,C,PLUGIN_IS_READY,D)
 extern bool plugin_foreach_with_mask(THD *thd, plugin_foreach_func *func,
                                      int type, uint state_mask, void *arg);
-
-/* interface to randomly access plugin data */
-struct st_plugin_int *plugin_find_by_type(LEX_STRING *plugin, int type);
+extern bool plugin_foreach_with_mask(THD *thd, plugin_foreach_func **funcs,
+                                     int type, uint state_mask, void *arg);
 int lock_plugin_data();
 int unlock_plugin_data();
 
+/**
+  Initialize one plugin.
+*/
+bool plugin_early_load_one(int *argc, char **argv, const char *plugin);
+
 /**
   Create deep copy of system_variables instance.
 */
