@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -334,7 +334,7 @@ bool trans_commit_implicit(THD *thd)
 #endif /* WITH_WSREP */
   }
   else if (tc_log)
-    tc_log->commit(thd, true);
+    res= tc_log->commit(thd, true);
 
   if (res == FALSE)
     if (thd->rpl_thd_ctx.session_gtids_ctx().
@@ -506,7 +506,7 @@ bool trans_commit_stmt(THD *thd)
 #endif /* WITH_WSREP */
   }
   else if (tc_log)
-    tc_log->commit(thd, false);
+    res= tc_log->commit(thd, false);
   if (res == FALSE && !thd->in_active_multi_stmt_transaction())
     if (thd->rpl_thd_ctx.session_gtids_ctx().
         notify_after_transaction_commit(thd))
