@@ -123,7 +123,9 @@ static wsrep_cb_status_t wsrep_apply_events(THD*        thd,
     case binary_log::FORMAT_DESCRIPTION_EVENT:
       wsrep_set_apply_format(thd, (Format_description_log_event*)ev);
       continue;
+    /* If gtid_mode=OFF then async master-slave will generate ANONYMOUS GTID. */
     case binary_log::GTID_LOG_EVENT:
+    case binary_log::ANONYMOUS_GTID_LOG_EVENT:
     {
       Gtid_log_event* gev= (Gtid_log_event*)ev;
       if (gev->get_gno() == 0)
@@ -141,6 +143,7 @@ static wsrep_cb_status_t wsrep_apply_events(THD*        thd,
       thd->variables.option_bits&= ~OPTION_BEGIN;
       thd->server_status&= ~SERVER_STATUS_IN_TRANS;
       assert(event == 1);
+      break;
     }
     default:
       break;
@@ -210,11 +213,24 @@ static wsrep_cb_status_t wsrep_apply_events(THD*        thd,
       DBUG_RETURN(WSREP_CB_FAILURE);
     }
 
-    /* row-query-log-event is generated to log extra information when following
-    configuration is set "binlog_rows_query_log_events". The event is cleared
-    after processing of ROWS_LOG_EVENT. Avoid removing it here. */
-    if (ev->get_type_code() != binary_log::ROWS_QUERY_LOG_EVENT)
+    switch (ev->get_type_code()) {
+    case binary_log::ROWS_QUERY_LOG_EVENT:
+      /*
+        Setting binlog_rows_query_log_events to ON will generate
+        ROW_QUERY_LOG_EVENT. This event logs an extra information while logging
+        row information and so event should be kept infact till ROW_LOG_EVENT
+        is processed and should be freed once ROW_LOG_EVENT is done.
+
+        Keeping Rows_log event, it will be needed still, and will be deleted later
+        in rli->cleanup_context()
+        Also FORMAT_DESCRIPTION_EVENT is needed further, but it skipped from this loop
+        by 'continue' above, and thus  avoids the following 'delete ev'
+      */
+      continue;
+    default:
       delete ev;
+      break;
+    }
   }
 
  error:
@@ -247,7 +263,8 @@ wsrep_cb_status_t wsrep_apply_cb(void* const             ctx,
                  {
                    const char act[]=
                      "now "
-                     "wait_for signal.wsrep_apply_cb";
+                     "SIGNAL sync.wsrep_apply_cb_reached "
+                     "WAIT_FOR signal.wsrep_apply_cb";
                    DBUG_ASSERT(!debug_sync_set_action(thd,
                                                       STRING_WITH_LEN(act)));
                  };);
@@ -318,6 +335,14 @@ static wsrep_cb_status_t wsrep_commit(THD* const thd)
   WSREP_DEBUG("%s", thd->wsrep_info);
   thd_proc_info(thd, thd->wsrep_info);
 
+  if (!thd->get_transaction()->is_empty(Transaction_ctx::STMT))
+  {
+    WSREP_INFO("Applier statement commit needed");
+    if (trans_commit_stmt(thd))
+    {
+      WSREP_WARN("Applier statement commit failed");
+    }
+  }
   wsrep_cb_status_t const rcode(trans_commit(thd) ?
                                 WSREP_CB_FAILURE : WSREP_CB_SUCCESS);
 
@@ -352,9 +377,14 @@ static wsrep_cb_status_t wsrep_rollback(THD* const thd)
   WSREP_DEBUG("%s", thd->wsrep_info);
   thd_proc_info(thd, thd->wsrep_info);
 
-  /* Check for comments in Relay_log_info::cleanup_context */
-  trans_rollback_stmt(thd);
-
+  if (!thd->get_transaction()->is_empty(Transaction_ctx::STMT))
+  {
+    WSREP_INFO("Applier statement rollback needed");
+    if (trans_rollback_stmt(thd))
+    {
+      WSREP_WARN("Applier statement rollback failed");
+    }
+  }
   wsrep_cb_status_t const rcode(trans_rollback(thd) ?
                                 WSREP_CB_FAILURE : WSREP_CB_SUCCESS);
 
