@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2000, 2017, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2000, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -18,10 +18,17 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 /* The InnoDB handler: the interface between MySQL and InnoDB. */
 
+#ifdef WITH_WSREP
+#include <wsrep_mysqld.h>
+#endif /* WITH_WSREP */
+
 /** "GEN_CLUST_INDEX" is the name reserved for InnoDB default
 system clustered index when there is no primary key. */
 extern const char innobase_index_reserve_name[];
 
+/* Deprecation warning text */
+extern const char PARTITION_IN_SHARED_TABLESPACE_WARNING[];
+
 /* "innodb_file_per_table" tablespace name  is reserved by InnoDB in order
 to explicitly create a file_per_table tablespace for the table. */
 extern const char reserved_file_per_table_space_name[];
@@ -92,7 +99,7 @@ public:
 
 	uint max_supported_key_length() const;
 
-	uint max_supported_key_part_length() const;
+	uint max_supported_key_part_length(HA_CREATE_INFO *create_info) const;
 
 	const key_map* keys_to_use_for_scanning();
 
@@ -438,7 +445,7 @@ public:
 	/* @} */
 
 #ifdef WITH_WSREP
-	int wsrep_append_keys(THD *thd, bool shared,
+	int wsrep_append_keys(THD *thd, wsrep_key_type key_type,
 				  const uchar* record0, const uchar* record1);
 #endif /* WITH_WSREP */
 
@@ -728,6 +735,18 @@ const HA_CREATE_INFO*	create_info)
 				reserved_system_space_name)));
 }
 
+/** Check if tablespace is shared tablespace.
+@param[in]      tablespace_name Name of the tablespace
+@return true if tablespace is a shared tablespace. */
+UNIV_INLINE
+bool is_shared_tablespace(const char *tablespace_name) {
+  if (tablespace_name != NULL && tablespace_name[0] != '\0' &&
+      (strcmp(tablespace_name, reserved_file_per_table_space_name) != 0)) {
+    return true;
+  }
+  return false;
+}
+
 /** Parse hint for table and its indexes, and update the information
 in dictionary.
 @param[in]	thd		Connection thread
@@ -853,6 +872,22 @@ public:
 		const char*     name,
 		ibool           set_lower_case);
 
+	/** If master key encryption is requested, check for master key availability
+	and set the encryption flag in table flags
+	@param[in,out]	table	table object
+	@return on success DB_SUCCESS else DB_UNSPPORTED on failure */
+
+	dberr_t	enable_master_key_encryption(dict_table_t*	table);
+
+	/** If keyring encryption is requested, check for tablespace's key availability
+	and set the encryption flag in table flags
+	@param[in,out]	table	table object
+	@param[in,out]  rotated_keys_encryption_option  contains appropriate
+			FIL_ENCRYPTION_(ON/DEFAULT/OFF)
+	@return on success DB_SUCCESS else DB_UNSPPORTED on failure */
+
+        dberr_t enable_keyring_encryption(dict_table_t *   table,fil_encryption_t &rotated_keys_encryption_option);
+
 private:
 	/** Parses the table name into normal name and either temp path or
 	remote path if needed.*/
