@@ -57,6 +57,7 @@ Created 2/16/1996 Heikki Tuuri
 #include "os0file.h"
 #include "os0thread.h"
 #include "fil0fil.h"
+#include "fil0crypt.h"
 #include "fsp0fsp.h"
 #include "rem0rec.h"
 #include "mtr0mtr.h"
@@ -69,6 +70,7 @@ Created 2/16/1996 Heikki Tuuri
 #include "trx0sys.h"
 #include "btr0btr.h"
 #include "btr0cur.h"
+#include "btr0scrub.h"
 #include "rem0rec.h"
 #include "ibuf0ibuf.h"
 #include "srv0start.h"
@@ -207,8 +209,16 @@ mysql_pfs_key_t	srv_worker_thread_key;
 
 #ifdef WITH_WSREP
 extern my_bool wsrep_recovery;
+
+#ifdef WITH_INNODB_DISALLOW_WRITES
+/* Must always init to FALSE. */
+static my_bool	innobase_disallow_writes	= FALSE;
+#endif /* WITH_INNODB_DISALLOW_WRITES */
+
 #endif /* WITH_WSREP */
 
+int unlock_keyrings(THD *thd);
+
 #ifdef HAVE_PSI_STAGE_INTERFACE
 /** Array of all InnoDB stage events for monitoring activities via
 performance schema. */
@@ -460,10 +470,31 @@ create_log_files(
 	fil_space_t*	log_space = fil_space_create(
 		"innodb_redo_log", SRV_LOG_SPACE_FIRST_ID,
 		fsp_flags_set_page_size(0, univ_page_size),
-		FIL_TYPE_LOG);
+		FIL_TYPE_LOG,
+                NULL);
 	ut_a(fil_validate());
 	ut_a(log_space != NULL);
 
+      /* Once the redo log is set to be encrypted,
+        initialize encryption information. */
+       if (srv_redo_log_encrypt != REDO_LOG_ENCRYPT_OFF) {
+               if (!Encryption::check_keyring()) {
+                       ib::error()
+                               << "Redo log encryption is enabled,"
+                               << " but keyring plugin is not loaded.";
+
+                       return(DB_ERROR);
+               }
+
+               log_space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+               err = fil_set_encryption(log_space->id,
+                                        Encryption::AES,
+                                        NULL,
+                                        NULL);
+               ut_ad(err == DB_SUCCESS);
+       }
+
+
 	logfile0 = fil_node_create(
 		logfilename, (ulint) srv_log_file_size,
 		log_space, false, false);
@@ -499,6 +530,16 @@ create_log_files(
 	recv_reset_logs(lsn);
 	log_mutex_exit();
 
+	/* Write encryption information into the first log file header
+	if redo log is set with encryption. */
+	if (FSP_FLAGS_GET_ENCRYPTION(log_space->flags)) {
+		if (!log_write_encryption(log_space->encryption_key,
+					  log_space->encryption_iv
+					  )) {
+			return(DB_ERROR);
+		}
+	}
+
 	return(DB_SUCCESS);
 }
 
@@ -637,6 +678,67 @@ srv_undo_tablespace_create(
 
 	return(err);
 }
+
+/** Try to read encryption metadata from an undo tablespace.
+@param[in]	fh		file handle of undo log file
+@param[in]	space		undo tablespace
+@return DB_SUCCESS if success */
+static
+dberr_t
+srv_undo_tablespace_read_encryption(
+	pfs_os_file_t	fh,
+	fil_space_t*	space)
+{
+	IORequest	request;
+	ulint		n_read = 0;
+	size_t		page_size = UNIV_PAGE_SIZE_MAX;
+	dberr_t		err = DB_ERROR;
+ 	byte* first_page_buf = static_cast<byte*>(
+		ut_malloc_nokey(2 * UNIV_PAGE_SIZE_MAX));
+	/* Align the memory for a possible read from a raw device */
+	byte* first_page = static_cast<byte*>(
+		ut_align(first_page_buf, UNIV_PAGE_SIZE));
+ 	/* Don't want unnecessary complaints about partial reads. */
+	request.disable_partial_io_warnings();
+ 	err = os_file_read_no_error_handling(
+		request, fh, first_page, 0, page_size, &n_read);
+ 	if (err != DB_SUCCESS) {
+		ib::info()
+			<< "Cannot read first page of '"
+			<< space->name << "' "
+			<< ut_strerr(err);
+		ut_free(first_page_buf);
+		return(err);
+	}
+ 	ulint			offset;
+	const page_size_t	space_page_size(space->flags);
+ 	offset = fsp_header_get_encryption_offset(space_page_size);
+	ut_ad(offset);
+ 	/* Return if the encryption metadata is empty. */
+	if (memcmp(first_page + offset,
+		   ENCRYPTION_KEY_MAGIC_V2,
+		   ENCRYPTION_MAGIC_SIZE) != 0) {
+		ut_free(first_page_buf);
+		return(DB_SUCCESS);
+	}
+ 	byte	key[ENCRYPTION_KEY_LEN];
+	byte	iv[ENCRYPTION_KEY_LEN];
+	if (fsp_header_get_encryption_key(space->flags, key,
+					  iv, first_page)) {
+ 		space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+		err = fil_set_encryption(space->id,
+					 Encryption::AES,
+					 key,
+					 iv);
+		ut_ad(err == DB_SUCCESS);
+	} else {
+		ut_free(first_page_buf);
+		return(DB_FAIL);
+	}
+ 	ut_free(first_page_buf);
+ 	return(DB_SUCCESS);
+}
+
 /*********************************************************************//**
 Open an undo tablespace.
 @return DB_SUCCESS or error code */
@@ -694,9 +796,6 @@ srv_undo_tablespace_open(
 		size = os_file_get_size(fh);
 		ut_a(size != (os_offset_t) -1);
 
-		ret = os_file_close(fh);
-		ut_a(ret);
-
 		/* Load the tablespace into InnoDB's internal
 		data structures. */
 
@@ -710,7 +809,7 @@ srv_undo_tablespace_open(
 		flags = fsp_flags_init(
 			univ_page_size, false, false, false, false);
 		space = fil_space_create(
-			undo_name, space_id, flags, FIL_TYPE_TABLESPACE);
+			undo_name, space_id, flags, FIL_TYPE_TABLESPACE, NULL);
 
 		ut_a(fil_validate());
 		ut_a(space);
@@ -721,11 +820,26 @@ srv_undo_tablespace_open(
 		is 64 bits. It is OK to cast the n_pages to ulint because
 		the unit has been scaled to pages and page number is always
 		32 bits. */
-		if (fil_node_create(
+		if (!fil_node_create(
 			name, (ulint) n_pages, space, false, atomic_write)) {
-
-			err = DB_SUCCESS;
+			os_file_close(fh);
+			ib::error() << "Error creating file node for " << undo_name;
+			return(DB_ERROR);
 		}
+
+		err = DB_SUCCESS;
+		/* Read the encryption metadata in this undo tablespace.
+		If the encryption info in the first page cannot be decrypted
+		by the master key, this table cannot be opened. */
+		err = srv_undo_tablespace_read_encryption(fh, space);
+ 		/* The file handle will no longer be needed. */
+		os_file_close(fh);
+
+		if (err != DB_SUCCESS) {
+			ib::error() << "Error reading encryption for " << undo_name;
+			return(err);
+		}
+
 	}
 
 	return(err);
@@ -1237,14 +1351,25 @@ srv_open_tmp_tablespace(
 
 			if (srv_tmp_tablespace_encrypt) {
 
+				/* Make sure the keyring is loaded. */
+				if (!Encryption::check_keyring()) {
+					srv_tmp_tablespace_encrypt = false;
+					ib::error() << "Can't set temporary"
+						<< " tablespace to be encrypted"
+						<< " because keyring plugin is"
+						<< " not available.";
+				        return(DB_ERROR);
+				}
+
 				fil_space_t*	space =
 					fil_space_get(temp_space_id);
-				space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
 
 				err = fil_set_encryption(space->id,
 							 Encryption::AES,
 							 NULL,
 							 NULL);
+
+				tmp_space->set_flags(space->flags);
 				ut_a(err == DB_SUCCESS);
 			}
 
@@ -1332,6 +1457,10 @@ srv_shutdown_all_bg_threads()
 				/* d. Wakeup purge threads. */
 				srv_purge_wakeup();
 			}
+
+			if (srv_n_fil_crypt_threads_started) {
+				os_event_set(fil_crypt_threads_event);
+			}
 		}
 
 		if (srv_start_state_is_set(SRV_START_STATE_IO)) {
@@ -1495,6 +1624,67 @@ srv_prepare_to_delete_redo_log_files(
 	return(flushed_lsn);
 }
 
+/** Enable encryption of system tablespace if requested. At
+startup load the encryption information from first datafile
+to tablespace object
+@return DB_SUCCESS on succes, others on failure */
+static
+dberr_t
+srv_sys_enable_encryption(bool create_new_db) {
+
+	fil_space_t*	space = fil_space_get(TRX_SYS_SPACE);
+	dberr_t		err = DB_SUCCESS;
+
+	if (create_new_db && srv_sys_tablespace_encrypt) {
+
+		space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+		srv_sys_space.set_flags(space->flags);
+
+		err = fil_set_encryption(space->id,
+					 Encryption::AES,
+					 NULL,
+					 NULL);
+		ut_ad(err == DB_SUCCESS);
+	} else {
+
+		ulint	fsp_flags = srv_sys_space.m_files.begin()->flags();
+		bool	is_encrypted = FSP_FLAGS_GET_ENCRYPTION(fsp_flags);
+
+		if (is_encrypted && !srv_sys_tablespace_encrypt) {
+			ib::error() << "The system tablespace is encrypted but"
+				<< " --innodb_sys_tablespace_encrypt is"
+				<< " OFF. Enable the option and start server";
+			return(DB_ERROR);
+		}
+
+		if (!is_encrypted && srv_sys_tablespace_encrypt) {
+			ib::error() << "The system tablespace is not encrypted but"
+				<< " --innodb_sys_tablespace_encrypt is"
+				<< " ON. This instance was not bootstrapped"
+				<< " with --innodb_sys_tablespace_encrypt=ON."
+				<< " Disable this option and start server";
+			return(DB_ERROR);
+		}
+
+		if (is_encrypted) {
+
+			space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+			srv_sys_space.set_flags(space->flags);
+
+			err = fil_set_encryption(
+				space->id,
+				Encryption::AES,
+				srv_sys_space.m_files.begin()->m_encryption_key,
+				srv_sys_space.m_files.begin()->m_encryption_iv);
+			ut_ad(err == DB_SUCCESS);
+
+			recv_sys->dblwr.decrypt_sys_dblwr_pages();
+		}
+	}
+
+	return(err);
+}
+
 /********************************************************************
 Starts InnoDB and creates a new database if database files
 are not found and the user wants.
@@ -1684,6 +1874,15 @@ innobase_start_or_create_for_mysql(void)
 		return(srv_init_abort(DB_ERROR));
 	}
 
+#ifdef WITH_WSREP
+	if (innobase_disallow_writes) {
+		ib::warn() << "innodb_disallow_writes has been deprecated and"
+			      " will be removed in future release."
+			      " Consider using read_only instead.";
+	}
+#endif /* WITH_WSREP */
+
+
 	/* Note that the call srv_boot() also changes the values of
 	some variables to the units used by InnoDB internally */
 
@@ -2009,6 +2208,10 @@ innobase_start_or_create_for_mysql(void)
 
 	switch (err) {
 	case DB_SUCCESS:
+		err = srv_sys_enable_encryption(create_new_db);
+		if (err != DB_SUCCESS) {
+			return(srv_init_abort(err));
+		}
 		break;
 	case DB_CANNOT_OPEN_FILE:
 		ib::error()
@@ -2154,7 +2357,7 @@ innobase_start_or_create_for_mysql(void)
 			"innodb_redo_log",
 			SRV_LOG_SPACE_FIRST_ID,
 			fsp_flags_set_page_size(0, univ_page_size),
-			FIL_TYPE_LOG);
+			FIL_TYPE_LOG, NULL);
 
 		ut_a(fil_validate());
 		ut_a(log_space);
@@ -2177,6 +2380,14 @@ innobase_start_or_create_for_mysql(void)
 				    SRV_LOG_SPACE_FIRST_ID)) {
 			return(srv_init_abort(DB_ERROR));
 		}
+
+		/* Read the first log file header to get the encryption
+		information if it exist. */
+		if (srv_force_recovery < SRV_FORCE_NO_LOG_REDO) {
+		    if (!log_read_encryption()) {
+			return(srv_init_abort(DB_ERROR));
+		    }
+		}
 	}
 
 files_checked:
@@ -2851,6 +3062,11 @@ files_checked:
 		/* Create the thread that will optimize the FTS sub-system. */
 		fts_optimize_init();
 
+		fil_system_enter();
+		btr_scrub_init();
+		fil_crypt_threads_init();
+		fil_system_exit();
+
 		srv_start_state_set(SRV_START_STATE_STAT);
 	}
 
@@ -2950,6 +3166,9 @@ innobase_shutdown_for_mysql(void)
 
 	if (!srv_read_only_mode) {
 		dict_stats_thread_deinit();
+		/* Shutdown key rotation threads */
+		fil_crypt_threads_cleanup();
+		btr_scrub_cleanup();
 	}
 
 	/* This must be disabled before closing the buffer pool
@@ -3008,6 +3227,8 @@ innobase_shutdown_for_mysql(void)
 	srv_was_started = FALSE;
 	srv_start_has_been_called = FALSE;
 
+	unlock_keyrings(NULL);
+
 	return(DB_SUCCESS);
 }
 #endif /* !UNIV_HOTBACKUP */
