@@ -131,8 +131,6 @@ trx_init(
 	trx_t::state here to NOT_STARTED. The FORCED_ROLLBACK
 	status is required for asynchronous handling. */
 
-	trx->id_saved = trx->id;
-
 	trx->id = 0;
 
 	trx->preallocated_id = 0;
@@ -200,14 +198,7 @@ trx_init(
 
 	trx->lock.table_cached = 0;
 
-	trx->io_reads = 0;
-	trx->io_read = 0;
-	trx->io_reads_wait_timer = 0;
-	trx->lock_que_wait_timer = 0;
-	trx->innodb_que_wait_timer = 0;
-	trx->distinct_page_access = 0;
-	trx->distinct_page_access_hash = NULL;
-	trx->take_stats = false;
+	trx->stats.set(false);
 
 	/* During asynchronous rollback, we should reset forced rollback flag
 	only after rollback is complete to avoid race with the thread owning
@@ -343,8 +334,6 @@ struct TrxFactory {
 
 		trx->lock.table_locks.~lock_pool_t();
 
-		ut_ad(!trx->distinct_page_access_hash);
-
 		trx->hit_list.~hit_list_t();
 	}
 
@@ -584,12 +573,6 @@ trx_allocate_for_mysql(void)
 
 	trx_sys_mutex_exit();
 
-	if (UNIV_UNLIKELY(trx->take_stats)) {
-		trx->distinct_page_access_hash
-			= static_cast<byte *>(ut_zalloc(DPAH_SIZE,
-				mem_key_trx_distinct_page_access_hash));
-	}
-
 	return(trx);
 }
 
@@ -648,12 +631,6 @@ trx_free_resurrected(trx_t* trx)
 void
 trx_free_for_background(trx_t* trx)
 {
-	if (trx->distinct_page_access_hash)
-	{
-		ut_free(trx->distinct_page_access_hash);
-		trx->distinct_page_access_hash= NULL;
-	}
-
 	trx_validate_state_before_free(trx);
 
 	trx_free(trx);
@@ -705,12 +682,6 @@ trx_disconnect_from_mysql(
 	trx_t*	trx,
 	bool	prepared)
 {
-	if (trx->distinct_page_access_hash)
-	{
-		ut_free(trx->distinct_page_access_hash);
-		trx->distinct_page_access_hash= NULL;
-	}
-
 	trx_sys_mutex_enter();
 
 	ut_ad(trx->in_mysql_trx_list);
@@ -831,7 +802,7 @@ trx_resurrect_table_locks(
 	     i != tables.end(); i++) {
 		if (dict_table_t* table = dict_table_open_on_id(
 			    *i, FALSE, DICT_TABLE_OP_LOAD_TABLESPACE)) {
-			if (table->ibd_file_missing
+			if (table->file_unreadable
 			    || dict_table_is_temporary(table)) {
 				mutex_enter(&dict_sys->mutex);
 				dict_table_close(table, TRUE, FALSE);
@@ -1776,12 +1747,13 @@ trx_write_serialisation_history(
         {
             trx_sys_update_wsrep_checkpoint(trx->xid, sys_header, mtr);
         }
-	else if (trx->wsrep_recover_xid)
+	else if (trx->wsrep_recover_xid
+		 && wsrep_is_wsrep_xid(trx->wsrep_recover_xid))
 	{
 		trx_sys_update_wsrep_checkpoint(
 				trx->wsrep_recover_xid, sys_header, mtr, true);
-		trx->wsrep_recover_xid = NULL;
 	}
+	trx->wsrep_recover_xid = NULL;
 #endif /* WITH_WSREP */
 
 	/* Update the latest MySQL binlog name and offset info
@@ -2206,11 +2178,6 @@ trx_commit_in_memory(
 		trx->lock.was_chosen_as_deadlock_victim = FALSE;
 	}
 #endif /* WITH_WSREP */
-	if (UNIV_LIKELY_NULL(trx->distinct_page_access_hash)) {
-
-		ut_free(trx->distinct_page_access_hash);
-		trx->distinct_page_access_hash= NULL;
-	}
 
 	/* trx->in_mysql_trx_list would hold between
 	trx_allocate_for_mysql() and trx_free_for_mysql(). It does not
@@ -2482,21 +2449,11 @@ trx_commit_or_rollback_prepare(
 
 		if (trx->lock.que_state == TRX_QUE_LOCK_WAIT) {
 
-			ulint		sec;
-			ulint		ms;
-			ib_uint64_t	now;
-
 			ut_a(trx->lock.wait_thr != NULL);
 			trx->lock.wait_thr->state = QUE_THR_SUSPENDED;
 			trx->lock.wait_thr = NULL;
 
-			if (UNIV_UNLIKELY(trx->take_stats)) {
-				ut_usectime(&sec, &ms);
-				now = (ib_uint64_t)sec * 1000000 + ms;
-				trx->lock_que_wait_timer
-					+= (ulint)
-					(now - trx->lock_que_wait_ustarted);
-			}
+			trx->stats.stop_lock_wait(*trx);
 
 			trx->lock.que_state = TRX_QUE_RUNNING;
 		}
@@ -3568,6 +3525,7 @@ trx_set_rw_mode(
 	ut_ad(trx->rsegs.m_redo.rseg == 0);
 	ut_ad(!trx->in_rw_trx_list);
 	ut_ad(!trx_is_autocommit_non_locking(trx));
+	ut_ad(!trx->read_only);
 
 	if (srv_force_recovery >= SRV_FORCE_NO_TRX_UNDO) {
 		return;
@@ -3605,11 +3563,9 @@ trx_set_rw_mode(
 	}
 #endif /* UNIV_DEBUG */
 
-	if (!trx->read_only) {
-		UT_LIST_ADD_FIRST(trx_sys->rw_trx_list, trx);
+	UT_LIST_ADD_FIRST(trx_sys->rw_trx_list, trx);
 
-		ut_d(trx->in_rw_trx_list = true);
-	}
+	ut_d(trx->in_rw_trx_list = true);
 
 	mutex_exit(&trx_sys->mutex);
 }
