@@ -22,43 +22,48 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 ======= */
 
-#ident "Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved."
+#ident \
+    "Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved."
 
 #include "hatoku_hton.h"
+#include "src/ydb.h"
 
 #include <dlfcn.h>
 
 #include "my_tree.h"
+#include "sql_partition.h"
 
 #define TOKU_METADB_NAME "tokudb_meta"
 
+#if defined(HAVE_PSI_MUTEX_INTERFACE)
+static pfs_key_t tokudb_map_mutex_key;
+
+static PSI_mutex_info all_tokudb_mutexes[] = {
+    {&tokudb_map_mutex_key, "tokudb_map_mutex", 0},
+    {&ha_tokudb_mutex_key, "ha_tokudb_mutex", 0},
+};
+
+static PSI_rwlock_info all_tokudb_rwlocks[] = {
+    {&num_DBs_lock_key, "num_DBs_lock", 0},
+};
+#endif /* HAVE_PSI_MUTEX_INTERFACE */
+
 typedef struct savepoint_info {
     DB_TXN* txn;
     tokudb_trx_data* trx;
     bool in_sub_stmt;
 } *SP_INFO, SP_INFO_T;
 
-#if TOKU_INCLUDE_OPTION_STRUCTS
-ha_create_table_option tokudb_table_options[] = {
-    HA_TOPTION_SYSVAR("compression", row_format, row_format),
-    HA_TOPTION_END
-};
+static handler* tokudb_create_handler(handlerton* hton,
+                                      TABLE_SHARE* table,
+                                      MEM_ROOT* mem_root);
 
-ha_create_table_option tokudb_index_options[] = {
-    HA_IOPTION_BOOL("clustering", clustering, 0),
-    HA_IOPTION_END
-};
-#endif
+/** Return partitioning flags. */
+static uint tokudb_partition_flags();
 
-static handler* tokudb_create_handler(
-    handlerton* hton,
-    TABLE_SHARE* table,
-    MEM_ROOT* mem_root);
-
-static void tokudb_print_error(
-    const DB_ENV* db_env,
-    const char* db_errpfx,
-    const char* buffer);
+static void tokudb_print_error(const DB_ENV* db_env,
+                               const char* db_errpfx,
+                               const char* buffer);
 static void tokudb_cleanup_log_files(void);
 static int tokudb_end(handlerton* hton, ha_panic_function type);
 static bool tokudb_flush_logs(handlerton* hton, bool binlog_group_commit);
@@ -67,10 +72,8 @@ static bool tokudb_show_status(
     THD* thd,
     stat_print_fn* print,
     enum ha_stat_type);
-#if TOKU_INCLUDE_HANDLERTON_HANDLE_FATAL_SIGNAL
-static void tokudb_handle_fatal_signal(handlerton* hton, THD* thd, int sig);
-#endif
 static int tokudb_close_connection(handlerton* hton, THD* thd);
+static void tokudb_kill_connection(handlerton *hton, THD *thd);
 static int tokudb_commit(handlerton* hton, THD* thd, bool all);
 static int tokudb_rollback(handlerton* hton, THD* thd, bool all);
 static int tokudb_xa_prepare(handlerton* hton, THD* thd, bool all);
@@ -86,13 +89,7 @@ static int tokudb_release_savepoint(
     handlerton* hton,
     THD* thd,
     void* savepoint);
-#if 100000 <= MYSQL_VERSION_ID && MYSQL_VERSION_ID <= 100099
-static int tokudb_discover_table(handlerton *hton, THD* thd, TABLE_SHARE *ts);
-static int tokudb_discover_table_existence(
-    handlerton* hton,
-    const char* db,
-    const char* name);
-#endif
+#if defined(TOKU_INCLUDE_DISCOVER_FRM) && TOKU_INCLUDE_DISCOVER_FRM
 static int tokudb_discover(
     handlerton* hton,
     THD* thd,
@@ -116,26 +113,23 @@ static int tokudb_discover3(
     char* path,
     uchar** frmblob,
     size_t* frmlen);
+#endif  // defined(TOKU_INCLUDE_DISCOVER_FRM) && TOKU_INCLUDE_DISCOVER_FRM
 handlerton* tokudb_hton;
 
 const char* ha_tokudb_ext = ".tokudb";
 DB_ENV* db_env;
 
-#if TOKU_THDVAR_MEMALLOC_BUG
+#if defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
 static tokudb::thread::mutex_t tokudb_map_mutex;
 static TREE tokudb_map;
 struct tokudb_map_pair {
     THD* thd;
     char *last_lock_timeout;
 };
-#if 50500 <= MYSQL_VERSION_ID && MYSQL_VERSION_ID <= 50599
-static int tokudb_map_pair_cmp(void *custom_arg, const void *a, const void *b) {
-#else
 static int tokudb_map_pair_cmp(
-    const void* custom_arg,
+    TOKUDB_UNUSED(const void* custom_arg),
     const void* a,
     const void* b) {
-#endif
 
     const struct tokudb_map_pair *a_key = (const struct tokudb_map_pair *) a;
     const struct tokudb_map_pair *b_key = (const struct tokudb_map_pair *) b;
@@ -146,7 +140,7 @@ static int tokudb_map_pair_cmp(
     else
         return 0;
 };
-#endif
+#endif  // defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
 
 static PARTITIONED_COUNTER tokudb_primary_key_bytes_inserted;
 void toku_hton_update_primary_key_bytes_inserted(uint64_t row_size) {
@@ -219,6 +213,10 @@ extern "C" {
 // use constructor and destructor functions to create and destroy
 // the lock before and after main(), respectively.
 int tokudb_hton_initialized;
+
+// tokudb_hton_initialized_lock can not be instrumented as it must be
+// initialized before mysql_mutex_register() call to protect
+// some globals from race condition.
 tokudb::thread::rwlock_t tokudb_hton_initialized_lock;
 
 static SHOW_VAR *toku_global_status_variables = NULL;
@@ -274,10 +272,23 @@ static int tokudb_init_func(void *p) {
     int r;
 
     // 3938: lock the handlerton's initialized status flag for writing
-    tokudb_hton_initialized_lock.lock_write();
+    rwlock_t_lock_write(tokudb_hton_initialized_lock);
+
+#ifdef HAVE_PSI_INTERFACE
+    /* Register TokuDB mutex keys with MySQL performance schema */
+    int count;
+
+    count = array_elements(all_tokudb_mutexes);
+    mysql_mutex_register("tokudb", all_tokudb_mutexes, count);
+
+    count = array_elements(all_tokudb_rwlocks);
+    mysql_rwlock_register("tokudb", all_tokudb_rwlocks, count);
+
+    tokudb_map_mutex.reinit(tokudb_map_mutex_key);
+#endif /* HAVE_PSI_INTERFACE */
 
     db_env = NULL;
-    tokudb_hton = (handlerton *) p;
+    tokudb_hton = (handlerton*)p;
 
     if (tokudb::sysvars::check_jemalloc) {
         typedef int (*mallctl_type)(
@@ -343,22 +354,22 @@ static int tokudb_init_func(void *p) {
 #endif
 
     tokudb_hton->create = tokudb_create_handler;
+    if (tokudb::sysvars::enable_native_partition)
+        tokudb_hton->partition_flags = tokudb_partition_flags;
     tokudb_hton->close_connection = tokudb_close_connection;
+    tokudb_hton->kill_connection = tokudb_kill_connection;
 
     tokudb_hton->savepoint_offset = sizeof(SP_INFO_T);
     tokudb_hton->savepoint_set = tokudb_savepoint;
     tokudb_hton->savepoint_rollback = tokudb_rollback_to_savepoint;
     tokudb_hton->savepoint_release = tokudb_release_savepoint;
 
-#if 100000 <= MYSQL_VERSION_ID && MYSQL_VERSION_ID <= 100099
-    tokudb_hton->discover_table = tokudb_discover_table;
-    tokudb_hton->discover_table_existence = tokudb_discover_table_existence;
-#else
+#if defined(TOKU_INCLUDE_DISCOVER_FRM) && TOKU_INCLUDE_DISCOVER_FRM
     tokudb_hton->discover = tokudb_discover;
 #if defined(MYSQL_HANDLERTON_INCLUDE_DISCOVER2)
     tokudb_hton->discover2 = tokudb_discover2;
-#endif
-#endif
+#endif  // MYSQL_HANDLERTON_INCLUDE_DISCOVER2
+#endif  // defined(TOKU_INCLUDE_DISCOVER_FRM) && TOKU_INCLUDE_DISCOVER_FRM
     tokudb_hton->commit = tokudb_commit;
     tokudb_hton->rollback = tokudb_rollback;
     tokudb_hton->prepare = tokudb_xa_prepare;
@@ -369,14 +380,6 @@ static int tokudb_init_func(void *p) {
     tokudb_hton->panic = tokudb_end;
     tokudb_hton->flush_logs = tokudb_flush_logs;
     tokudb_hton->show_status = tokudb_show_status;
-#if TOKU_INCLUDE_HANDLERTON_HANDLE_FATAL_SIGNAL
-    tokudb_hton->handle_fatal_signal = tokudb_handle_fatal_signal;
-#endif
-
-#if TOKU_INCLUDE_OPTION_STRUCTS
-    tokudb_hton->table_options = tokudb_table_options;
-    tokudb_hton->index_options = tokudb_index_options;
-#endif
 
     if (!tokudb_home)
         tokudb_home = mysql_real_data_home;
@@ -532,14 +535,15 @@ static int tokudb_init_func(void *p) {
 
     db_env->set_update(db_env, tokudb_update_fun);
 
-    db_env_set_direct_io(tokudb::sysvars::directio == TRUE);
+    db_env_set_direct_io(tokudb::sysvars::directio);
 
     db_env_set_compress_buffers_before_eviction(
-        tokudb::sysvars::compress_buffers_before_eviction == TRUE);
+        tokudb::sysvars::compress_buffers_before_eviction);
 
     db_env->change_fsync_log_period(db_env, tokudb::sysvars::fsync_log_period);
 
     db_env->set_lock_timeout_callback(db_env, tokudb_lock_timeout_callback);
+    db_env->set_dir_per_db(db_env, tokudb::sysvars::dir_per_db);
 
     db_env->set_loader_memory_size(
         db_env,
@@ -613,9 +617,9 @@ static int tokudb_init_func(void *p) {
 
     tokudb_primary_key_bytes_inserted = create_partitioned_counter();
 
-#if TOKU_THDVAR_MEMALLOC_BUG
+#if defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
     init_tree(&tokudb_map, 0, 0, 0, tokudb_map_pair_cmp, true, NULL, NULL);
-#endif
+#endif  // defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
 
     if (tokudb::sysvars::strip_frm_data) {
         r = tokudb::metadata::strip_frm_data(db_env);
@@ -644,23 +648,40 @@ error:
     DBUG_RETURN(true);
 }
 
-static int tokudb_done_func(void* p) {
+static int tokudb_done_func(TOKUDB_UNUSED(void* p)) {
     TOKUDB_DBUG_ENTER("");
     tokudb::memory::free(toku_global_status_variables);
     toku_global_status_variables = NULL;
     tokudb::memory::free(toku_global_status_rows);
     toku_global_status_rows = NULL;
+    tokudb_map_mutex.deinit();
+    toku_ydb_destroy();
     TOKUDB_DBUG_RETURN(0);
 }
 
-static handler* tokudb_create_handler(
-    handlerton* hton,
-    TABLE_SHARE* table,
-    MEM_ROOT* mem_root) {
-    return new(mem_root) ha_tokudb(hton, table);
+static handler* tokudb_create_handler(handlerton* hton,
+                                      TABLE_SHARE* table,
+                                      MEM_ROOT* mem_root) {
+    if (tokudb::sysvars::enable_native_partition &&
+        table && table->db_type() == tokudb_hton && table->partition_info_str &&
+        table->partition_info_str_len) {
+        ha_tokupart* file = new (mem_root) ha_tokupart(hton, table);
+        if (file && file->init_partitioning(mem_root)) {
+            delete file;
+            return nullptr;
+        }
+        return (file);
+    }
+
+    return new (mem_root) ha_tokudb(hton, table);
 }
 
-int tokudb_end(handlerton* hton, ha_panic_function type) {
+static uint tokudb_partition_flags() {
+    return (HA_CAN_EXCHANGE_PARTITION | HA_CANNOT_PARTITION_FK);
+}
+
+int tokudb_end(TOKUDB_UNUSED(handlerton* hton),
+               TOKUDB_UNUSED(ha_panic_function type)) {
     TOKUDB_DBUG_ENTER("");
     int error = 0;
     
@@ -668,7 +689,7 @@ int tokudb_end(handlerton* hton, ha_panic_function type) {
     // initialized. grab a writer lock for the duration of the
     // call, so we can drop the flag and destroy the mutexes
     // in isolation.
-    tokudb_hton_initialized_lock.lock_write();
+    rwlock_t_lock_write(tokudb_hton_initialized_lock);
     assert_always(tokudb_hton_initialized);
 
     tokudb::background::destroy();
@@ -725,9 +746,9 @@ int tokudb_end(handlerton* hton, ha_panic_function type) {
         tokudb_primary_key_bytes_inserted = NULL;
     }
 
-#if TOKU_THDVAR_MEMALLOC_BUG
+#if defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
     delete_tree(&tokudb_map);
-#endif
+#endif  // defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
 
     // 3938: drop the initialized flag and unlock
     tokudb_hton_initialized = 0;
@@ -736,37 +757,43 @@ int tokudb_end(handlerton* hton, ha_panic_function type) {
     TOKUDB_DBUG_RETURN(error);
 }
 
-static int tokudb_close_connection(handlerton* hton, THD* thd) {
+static int tokudb_close_connection(TOKUDB_UNUSED(handlerton* hton), THD* thd) {
     int error = 0;
     tokudb_trx_data* trx = (tokudb_trx_data*)thd_get_ha_data(thd, tokudb_hton);
     if (trx && trx->checkpoint_lock_taken) {
         error = db_env->checkpointing_resume(db_env);
     }
     tokudb::memory::free(trx);
-#if TOKU_THDVAR_MEMALLOC_BUG
-    tokudb_map_mutex.lock();
-    struct tokudb_map_pair key = { thd, NULL };
+#if defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
+    mutex_t_lock(tokudb_map_mutex);
+    struct tokudb_map_pair key = {thd, NULL};
     struct tokudb_map_pair* found_key =
-        (struct tokudb_map_pair*) tree_search(&tokudb_map, &key, NULL);
+        (struct tokudb_map_pair*)tree_search(&tokudb_map, &key, NULL);
 
     if (found_key) {
         tokudb::memory::free(found_key->last_lock_timeout);
         tree_delete(&tokudb_map, found_key, sizeof(*found_key), NULL);
     }
-    tokudb_map_mutex.unlock();
-#endif
+    mutex_t_unlock(tokudb_map_mutex);
+#endif  // defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
     return error;
 }
 
-bool tokudb_flush_logs(handlerton * hton, bool binlog_group_commit) {
+void tokudb_kill_connection(TOKUDB_UNUSED(handlerton* hton), THD* thd) {
+    TOKUDB_DBUG_ENTER("");
+    db_env->kill_waiter(db_env, thd);
+    DBUG_VOID_RETURN;
+}
+
+bool tokudb_flush_logs(TOKUDB_UNUSED(handlerton* hton),
+                       bool binlog_group_commit) {
     TOKUDB_DBUG_ENTER("");
     int error;
     bool result = 0;
 
-    if (tokudb::sysvars::checkpoint_on_flush_logs) {
-        //
-        // take the checkpoint
-        //
+    // if we are in 'FLUSH LOGS' and we are directed to checkpoint, do a
+    // checkpoint which also has the effect of flushing logs
+    if (!binlog_group_commit && tokudb::sysvars::checkpoint_on_flush_logs) {
         error = db_env->txn_checkpoint(db_env, 0, 0, 0);
         if (error) {
             my_error(ER_ERROR_DURING_CHECKPOINT, MYF(0), error);
@@ -774,12 +801,15 @@ bool tokudb_flush_logs(handlerton * hton, bool binlog_group_commit) {
             goto exit;
         }
     }
-    else {
+    // if we are either in 'FLUSH LOGS', or, we are not in 'FLUSH LOGS' but in
+    // binlog_group_commit and we are in high durability, flush 'em
+    else if (!binlog_group_commit ||
+             (tokudb::sysvars::fsync_log_period == 0 &&
+              tokudb::sysvars::commit_sync(NULL))) {
         error = db_env->log_flush(db_env, NULL);
         assert_always(error == 0);
     }
 
-    result = 0;
 exit:
     TOKUDB_DBUG_RETURN(result);
 }
@@ -844,23 +874,14 @@ static void tokudb_cleanup_handlers(tokudb_trx_data *trx, DB_TXN *txn) {
     }
 }
 
-#if MYSQL_VERSION_ID >= 50600
 extern "C" enum durability_properties thd_get_durability_property(
     const MYSQL_THD thd);
-#endif
 
 // Determine if an fsync is used when a transaction is committed.  
-static bool tokudb_sync_on_commit(THD* thd, tokudb_trx_data* trx, DB_TXN* txn) {
-#if MYSQL_VERSION_ID >= 50600
+static bool tokudb_sync_on_commit(THD* thd) {
     // Check the client durability property which is set during 2PC
     if (thd_get_durability_property(thd) == HA_IGNORE_DURABILITY)
         return false;
-#endif
-#if defined(MARIADB_BASE_VERSION)
-    // Check is the txn is prepared and the binlog is open
-    if (txn->is_prepared(txn) && mysql_bin_log.is_open())
-        return false;
-#endif
     if (tokudb::sysvars::fsync_log_period > 0)
         return false;
     return tokudb::sysvars::commit_sync(thd) != 0;
@@ -873,8 +894,7 @@ static int tokudb_commit(handlerton * hton, THD * thd, bool all) {
     DB_TXN **txn = all ? &trx->all : &trx->stmt;
     DB_TXN *this_txn = *txn;
     if (this_txn) {
-        uint32_t syncflag =
-            tokudb_sync_on_commit(thd, trx, this_txn) ? 0 : DB_TXN_NOSYNC;
+        uint32_t syncflag = tokudb_sync_on_commit(thd) ? 0 : DB_TXN_NOSYNC;
         TOKUDB_TRACE_FOR_FLAGS(
             TOKUDB_DEBUG_TXN,
             "commit trx %u txn %p syncflag %u",
@@ -942,7 +962,7 @@ static bool tokudb_sync_on_prepare(THD* thd) {
 }   
 
 static int tokudb_xa_prepare(handlerton* hton, THD* thd, bool all) {
-    TOKUDB_DBUG_ENTER("");
+    TOKUDB_DBUG_ENTER("%u", all);
     TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "enter");
     int r = 0;
 
@@ -970,6 +990,22 @@ static int tokudb_xa_prepare(handlerton* hton, THD* thd, bool all) {
         r = txn->xa_prepare(txn, &thd_xid, syncflag);
         // test hook to induce a crash on a debug build
         DBUG_EXECUTE_IF("tokudb_crash_prepare_after", DBUG_SUICIDE(););
+
+        // XA log entries can be interleaved in the binlog since XA prepare on the master
+        // flushes to the binlog.  There can be log entries from different clients pushed
+        // into the binlog before XA commit is executed on the master.  Therefore, the slave
+        // thread must be able to juggle multiple XA transactions.  Tokudb does this by
+        // zapping the client transaction context on the slave when executing the XA prepare
+        // and expecting to process XA commit with commit_by_xid (which supplies the XID so
+        // that the transaction can be looked up and committed).
+        if (r == 0 && all && thd->slave_thread) {
+            TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "zap txn context %u", thd_sql_command(thd));
+            if (thd_sql_command(thd) == SQLCOM_XA_PREPARE) {
+                trx->all = NULL;
+                trx->sub_sp_level = NULL;
+                trx->sp_level = NULL;
+            }
+        }
     } else {
         TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "nothing to prepare %d", all);
     }
@@ -977,7 +1013,9 @@ static int tokudb_xa_prepare(handlerton* hton, THD* thd, bool all) {
     TOKUDB_DBUG_RETURN(r);
 }
 
-static int tokudb_xa_recover(handlerton* hton, XID* xid_list, uint len) {
+static int tokudb_xa_recover(TOKUDB_UNUSED(handlerton* hton),
+                             XID* xid_list,
+                             uint len) {
     TOKUDB_DBUG_ENTER("");
     TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "enter");
     int r = 0;
@@ -997,9 +1035,10 @@ static int tokudb_xa_recover(handlerton* hton, XID* xid_list, uint len) {
     TOKUDB_DBUG_RETURN((int)num_returned);
 }
 
-static int tokudb_commit_by_xid(handlerton* hton, XID* xid) {
+static int tokudb_commit_by_xid(TOKUDB_UNUSED(handlerton* hton), XID* xid) {
     TOKUDB_DBUG_ENTER("");
     TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "enter");
+    TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "xid %p", xid);
     int r = 0;
     DB_TXN* txn = NULL;
     TOKU_XA_XID* toku_xid = (TOKU_XA_XID*)xid;
@@ -1016,9 +1055,10 @@ cleanup:
     TOKUDB_DBUG_RETURN(r);
 }
 
-static int tokudb_rollback_by_xid(handlerton* hton, XID*  xid) {
+static int tokudb_rollback_by_xid(TOKUDB_UNUSED(handlerton* hton), XID* xid) {
     TOKUDB_DBUG_ENTER("");
     TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "enter");
+    TOKUDB_TRACE_FOR_FLAGS(TOKUDB_DEBUG_XA, "xid %p", xid);
     int r = 0;
     DB_TXN* txn = NULL;
     TOKU_XA_XID* toku_xid = (TOKU_XA_XID*)xid;
@@ -1129,39 +1169,7 @@ static int tokudb_release_savepoint(
     TOKUDB_DBUG_RETURN(error);
 }
 
-#if 100000 <= MYSQL_VERSION_ID && MYSQL_VERSION_ID <= 100099
-static int tokudb_discover_table(handlerton *hton, THD* thd, TABLE_SHARE *ts) {
-    uchar *frmblob = 0;
-    size_t frmlen;
-    int res= tokudb_discover3(
-        hton,
-        thd,
-        ts->db.str,
-        ts->table_name.str,
-        ts->normalized_path.str,
-        &frmblob,
-        &frmlen);
-    if (!res)
-        res= ts->init_from_binary_frm_image(thd, true, frmblob, frmlen);
-    
-    my_free(frmblob);
-    // discover_table should returns HA_ERR_NO_SUCH_TABLE for "not exists"
-    return res == ENOENT ? HA_ERR_NO_SUCH_TABLE : res;
-}
-
-static int tokudb_discover_table_existence(
-    handlerton* hton,
-    const char* db,
-    const char* name) {
-
-    uchar *frmblob = 0;
-    size_t frmlen;
-    int res= tokudb_discover(hton, current_thd, db, name, &frmblob, &frmlen);
-    my_free(frmblob);
-    return res != ENOENT;
-}
-#endif
-
+#if defined(TOKU_INCLUDE_DISCOVER_FRM) && TOKU_INCLUDE_DISCOVER_FRM
 static int tokudb_discover(
     handlerton* hton,
     THD* thd,
@@ -1193,15 +1201,13 @@ static int tokudb_discover2(
     return tokudb_discover3(hton, thd, db, name, path, frmblob, frmlen);
 }
 
-static int tokudb_discover3(
-    handlerton* hton,
-    THD* thd,
-    const char* db,
-    const char* name,
-    char* path,
-    uchar** frmblob,
-    size_t* frmlen) {
-
+static int tokudb_discover3(TOKUDB_UNUSED(handlerton* hton),
+                            THD* thd,
+                            const char* db,
+                            const char* name,
+                            char* path,
+                            uchar** frmblob,
+                            size_t* frmlen) {
     TOKUDB_DBUG_ENTER("%s %s %s", db, name, path);
     int error;
     DB* status_db = NULL;
@@ -1211,23 +1217,9 @@ static int tokudb_discover3(
     DBT value = {};
     bool do_commit = false;
 
-#if 100000 <= MYSQL_VERSION_ID && MYSQL_VERSION_ID <= 100099
-    tokudb_trx_data* trx = (tokudb_trx_data*)thd_get_ha_data(thd, tokudb_hton);
-    if (thd_sql_command(thd) == SQLCOM_CREATE_TABLE &&
-        trx &&
-        trx->sub_sp_level) {
-        do_commit = false;
-        txn = trx->sub_sp_level;
-    } else {
-        error = txn_begin(db_env, 0, &txn, 0, thd);
-        if (error) { goto cleanup; }
-        do_commit = true;
-    }
-#else
     error = txn_begin(db_env, 0, &txn, 0, thd);
     if (error) { goto cleanup; }
     do_commit = true;
-#endif
 
     error = open_status_dictionary(&status_db, path, txn);
     if (error) { goto cleanup; }
@@ -1259,6 +1251,7 @@ cleanup:
     }
     TOKUDB_DBUG_RETURN(error);
 }
+#endif  // defined(TOKU_INCLUDE_DISCOVER_FRM) && TOKU_INCLUDE_DISCOVER_FRM
 
 
 #define STATPRINT(legend, val) if (legend != NULL && val != NULL) \
@@ -1287,19 +1280,6 @@ static bool tokudb_show_engine_status(THD * thd, stat_print_fn * stat_print) {
     const int bufsiz = 1024;
     char buf[bufsiz];
 
-#if MYSQL_VERSION_ID < 50500
-    {
-        sys_var* version = intern_find_sys_var("version", 0, false);
-        snprintf(
-            buf,
-            bufsiz,
-            "%s",
-            version->value_ptr(thd,
-            (enum_var_type)0,
-            (LEX_STRING*)NULL));
-        STATPRINT("Version", buf);
-    }
-#endif
     error = db_env->get_engine_status_num_rows (db_env, &max_rows);
     TOKU_ENGINE_STATUS_ROW_S mystat[max_rows];
     error = db_env->get_engine_status(
@@ -1464,7 +1444,7 @@ cleanup:
 }
 
 static bool tokudb_show_status(
-    handlerton* hton,
+    TOKUDB_UNUSED(handlerton* hton),
     THD* thd,
     stat_print_fn* stat_print,
     enum ha_stat_type stat_type) {
@@ -1479,22 +1459,9 @@ static bool tokudb_show_status(
     return false;
 }
 
-#if TOKU_INCLUDE_HANDLERTON_HANDLE_FATAL_SIGNAL
-static void tokudb_handle_fatal_signal(
-    TOKUDB_UNUSED(handlerton* hton),
-    TOKUDB_UNUSD(THD* thd),
-    int sig) {
-
-    if (tokudb_gdb_on_fatal) {
-        db_env_try_gdb_stack_trace(tokudb_gdb_path);
-    }
-}
-#endif
-
-static void tokudb_print_error(
-    const DB_ENV* db_env,
-    const char* db_errpfx,
-    const char* buffer) {
+static void tokudb_print_error(TOKUDB_UNUSED(const DB_ENV* db_env),
+                               const char* db_errpfx,
+                               const char* buffer) {
     sql_print_error("%s: %s", db_errpfx, buffer);
 }
 
@@ -1561,51 +1528,7 @@ struct st_mysql_storage_engine tokudb_storage_engine = {
     MYSQL_HANDLERTON_INTERFACE_VERSION
 };
 
-#if TOKU_INCLUDE_LOCK_TIMEOUT_QUERY_STRING
-struct tokudb_search_txn_extra {
-    bool match_found;
-    uint64_t match_txn_id;
-    uint64_t match_client_id;
-};
-
-static int tokudb_search_txn_callback(
-    DB_TXN* txn,
-    iterate_row_locks_callback iterate_locks,
-    void* locks_extra,
-    void* extra) {
-
-    uint64_t txn_id = txn->id64(txn);
-    uint64_t client_id = txn->get_client_id(txn);
-    struct tokudb_search_txn_extra* e =
-        reinterpret_cast<struct tokudb_search_txn_extra*>(extra);
-    if (e->match_txn_id == txn_id) {
-        e->match_found = true;
-        e->match_client_id = client_id;
-        return 1;
-    }
-    return 0;
-}
-
-static bool tokudb_txn_id_to_client_id(
-    THD* thd,
-    uint64_t blocking_txnid,
-    uint64_t* blocking_client_id) {
-
-    struct tokudb_search_txn_extra e = {
-        false,
-        blocking_txnid,
-        0
-    };
-    db_env->iterate_live_transactions(db_env, tokudb_search_txn_callback, &e);
-    if (e.match_found) {
-        *blocking_client_id = e.match_client_id;
-    }
-    return e.match_found;
-}
-#endif
-
 static void tokudb_pretty_key(
-    const DB* db,
     const DBT* key,
     const char* default_key,
     String* out) {
@@ -1627,12 +1550,12 @@ static void tokudb_pretty_key(
     }
 }
 
-void tokudb_pretty_left_key(const DB* db, const DBT* key, String* out) {
-    tokudb_pretty_key(db, key, "-infinity", out);
+void tokudb_pretty_left_key(const DBT* key, String* out) {
+    tokudb_pretty_key(key, "-infinity", out);
 }
 
-void tokudb_pretty_right_key(const DB* db, const DBT* key, String* out) {
-    tokudb_pretty_key(db, key, "+infinity", out);
+void tokudb_pretty_right_key(const DBT* key, String* out) {
+    tokudb_pretty_key(key, "+infinity", out);
 }
 
 const char* tokudb_get_index_name(DB* db) {
@@ -1679,20 +1602,20 @@ static void tokudb_lock_timeout_callback(
         log_str.append_ulonglong(blocking_txnid);
         if (tokudb_equal_key(left_key, right_key)) {
             String key_str;
-            tokudb_pretty_key(db, left_key, "?", &key_str);
+            tokudb_pretty_key(left_key, "?", &key_str);
             log_str.append(", \"key\":");
             log_str.append("\"");
             log_str.append(key_str);
             log_str.append("\"");
         } else {
             String left_str;
-            tokudb_pretty_left_key(db, left_key, &left_str);
+            tokudb_pretty_left_key(left_key, &left_str);
             log_str.append(", \"key_left\":");
             log_str.append("\"");
             log_str.append(left_str);
             log_str.append("\"");
             String right_str;
-            tokudb_pretty_right_key(db, right_key, &right_str);
+            tokudb_pretty_right_key(right_key, &right_str);
             log_str.append(", \"key_right\":");
             log_str.append("\"");
             log_str.append(right_str);
@@ -1705,14 +1628,14 @@ static void tokudb_lock_timeout_callback(
             char* new_lock_timeout =
                 tokudb::memory::strdup(log_str.c_ptr(), MY_FAE);
             tokudb::sysvars::set_last_lock_timeout(thd, new_lock_timeout);
-#if TOKU_THDVAR_MEMALLOC_BUG
-            tokudb_map_mutex.lock();
-            struct tokudb_map_pair old_key = { thd, old_lock_timeout };
+#if defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
+            mutex_t_lock(tokudb_map_mutex);
+            struct tokudb_map_pair old_key = {thd, old_lock_timeout};
             tree_delete(&tokudb_map, &old_key, sizeof old_key, NULL);
-            struct tokudb_map_pair new_key = { thd, new_lock_timeout };
+            struct tokudb_map_pair new_key = {thd, new_lock_timeout};
             tree_insert(&tokudb_map, &new_key, sizeof new_key, NULL);
-            tokudb_map_mutex.unlock();
-#endif
+            mutex_t_unlock(tokudb_map_mutex);
+#endif  // defined(TOKU_THDVAR_MEMALLOC_BUG) && TOKU_THDVAR_MEMALLOC_BUG
             tokudb::memory::free(old_lock_timeout);
         }
         // dump to stderr
@@ -1728,27 +1651,6 @@ static void tokudb_lock_timeout_callback(
                 static_cast<uint64_t>(mysql_thread_id),
                 (int)qs.length,
                 qs.str);
-#if TOKU_INCLUDE_LOCK_TIMEOUT_QUERY_STRING
-            uint64_t blocking_thread_id = 0;
-            if (tokudb_txn_id_to_client_id(
-                    thd,
-                    blocking_txnid,
-                    &blocking_thread_id)) {
-
-                String blocking_qs;
-                if (get_thread_query_string(
-                        blocking_thread_id,
-                        blocking_qs) == 0) {
-
-                    sql_print_error(
-                        "%s: blocking_thread_id:%" PRIu64 " q:%.*s",
-                        tokudb_hton_name,
-                        blocking_thread_id,
-                        blocking_qs.length(),
-                        blocking_qs.c_ptr());
-                }
-            }
-#endif
         }
     }
 }
@@ -1756,7 +1658,9 @@ static void tokudb_lock_timeout_callback(
 // Retrieves variables for information_schema.global_status.
 // Names (columnname) are automatically converted to upper case,
 // and prefixed with "TOKUDB_"
-static int show_tokudb_vars(THD *thd, SHOW_VAR *var, char *buff) {
+static int show_tokudb_vars(TOKUDB_UNUSED(THD* thd),
+                            SHOW_VAR* var,
+                            TOKUDB_UNUSED(char* buff)) {
     TOKUDB_DBUG_ENTER("");
 
     int error;
@@ -1863,21 +1767,7 @@ static SHOW_VAR toku_global_status_variables_export[]= {
     {NullS, NullS, SHOW_LONG, SHOW_SCOPE_GLOBAL}
 };
 
-#if TOKU_INCLUDE_BACKTRACE
-#include <execinfo.h>
-static void tokudb_backtrace(void) {
-    const int N_POINTERS = 30;
-    void *backtrace_pointers[N_POINTERS];
-    int n = backtrace(backtrace_pointers, N_POINTERS);
-    backtrace_symbols_fd(backtrace_pointers, n, fileno(stderr));
-}
-#endif
-
-#ifdef MARIA_PLUGIN_INTERFACE_VERSION
-maria_declare_plugin(tokudb) 
-#else
 mysql_declare_plugin(tokudb) 
-#endif
     {
         MYSQL_STORAGE_ENGINE_PLUGIN,
         &tokudb_storage_engine,
@@ -1890,13 +1780,8 @@ mysql_declare_plugin(tokudb)
         TOKUDB_PLUGIN_VERSION,
         toku_global_status_variables_export,  /* status variables */
         tokudb::sysvars::system_variables,   /* system variables */
-#ifdef MARIA_PLUGIN_INTERFACE_VERSION
-        tokudb::sysvars::version,
-        MariaDB_PLUGIN_MATURITY_STABLE /* maturity */
-#else
         NULL,                      /* config options */
         0,                         /* flags */
-#endif
     },
     tokudb::information_schema::trx,
     tokudb::information_schema::lock_waits,
@@ -1905,8 +1790,4 @@ mysql_declare_plugin(tokudb)
     tokudb::information_schema::fractal_tree_info,
     tokudb::information_schema::fractal_tree_block_map,
     tokudb::information_schema::background_job_status
-#ifdef MARIA_PLUGIN_INTERFACE_VERSION
-maria_declare_plugin_end;
-#else
 mysql_declare_plugin_end;
-#endif
