@@ -72,6 +72,7 @@ extern my_bool      compress_buffers_before_eviction;
 extern char*        data_dir;
 extern ulong        debug;
 extern my_bool      directio;
+extern my_bool      enable_native_partition;
 extern my_bool      enable_partial_eviction;
 extern int          fs_reserve_percent;
 extern uint         fsync_log_period;
@@ -81,19 +82,14 @@ extern uint         read_status_frequency;
 extern my_bool      strip_frm_data;
 extern char*        tmp_dir;
 extern uint         write_status_frequency;
+extern my_bool      dir_per_db;
 extern char*        version;
-
-#if TOKU_INCLUDE_HANDLERTON_HANDLE_FATAL_SIGNAL
-extern char*        gdb_path;
-extern my_bool      gdb_on_fatal;
-#endif
-
 extern my_bool         check_jemalloc;
 
-#if TOKUDB_DEBUG
+#if defined(TOKUDB_DEBUG)
 // used to control background job manager
 extern my_bool      debug_pause_background_job_manager;
-#endif // TOKUDB_DEBUG
+#endif // defined(TOKUDB_DEBUG)
 
 // session/thread
 my_bool     alter_print_error(THD* thd);
@@ -110,10 +106,10 @@ my_bool     create_index_online(THD* thd);
 my_bool     disable_hot_alter(THD* thd);
 my_bool     disable_prefetching(THD* thd);
 my_bool     disable_slow_alter(THD* thd);
-#if TOKU_INCLUDE_UPSERT
-my_bool     disable_slow_update(THD* thd);
-my_bool     disable_slow_upsert(THD* thd);
-#endif
+#if defined(TOKU_INCLUDE_UPSERT) && TOKU_INCLUDE_UPSERT
+my_bool     enable_fast_update(THD* thd);
+my_bool     enable_fast_upsert(THD* thd);
+#endif  // defined(TOKU_INCLUDE_UPSERT) && TOKU_INCLUDE_UPSERT
 empty_scan_mode_t empty_scan(THD* thd);
 uint        fanout(THD* thd);
 my_bool     hide_default_row_format(THD* thd);
@@ -132,11 +128,13 @@ my_bool     prelock_empty(THD* thd);
 uint        read_block_size(THD* thd);
 uint        read_buf_size(THD* thd);
 row_format_t row_format(THD *thd);
+#if defined(TOKU_INCLUDE_RFR) && TOKU_INCLUDE_RFR
 my_bool     rpl_check_readonly(THD* thd);
 my_bool     rpl_lookup_rows(THD* thd);
 ulonglong   rpl_lookup_rows_delay(THD* thd);
 my_bool     rpl_unique_checks(THD* thd);
 ulonglong   rpl_unique_checks_delay(THD* thd);
+#endif // defined(TOKU_INCLUDE_RFR) && TOKU_INCLUDE_RFR
 my_bool     support_xa(THD* thd);
 void        set_support_xa(THD* thd, my_bool xa);
 
